﻿//----------------------------------------------------------------------------------------------
//	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
//	This class can be used by anyone provided that the copyright notice remains intact.
//	Program developed by ABED Toufik.
//
//	This class helper is used to operate SIGMAKOKI Controllers FC series. 
//	Use the class to build your own program.
//	We can provide support for the other controllers.
//	Contact us : Phone (+ 81-3-5638-8228) or sales@sigma-koki.com.
//----------------------------------------------------------------------------------------------


using System;
using System.IO.Ports;
using System.Threading;
using System.Windows.Forms;
public partial class SKSampleClass
{
    public enum Controller_Type
    {
        FC_911,
        FC_611,
        FC_511,
        FC_411,
        FC_111
    }

    public SerialPort SK_SerialPort;
    private Controller_Type Controller;
    private int MaxAxis;
    private double Resol;      // Resolution of controller 
    private double MaxSpeed;   // Max speed of controller, note that Max speed value of stages may differs than the Controller
    private double MinSpeed;   // Min speed of controller, note that Min speed value of stages may differs than the Controller
    private string ErrMsg;
    private string ErrTitle;
    private Label PosText;
    public bool Language;

    public SKSampleClass()
    {
        SK_SerialPort = new SerialPort();
        SK_SerialPort.PortName = "COM1";     // Default value
        SK_SerialPort.BaudRate = 9600;       // Default value
        SK_SerialPort.NewLine = "\r\n"; ;      // Default value
        SK_SerialPort.ReadTimeout = 5000;    // Default value
        SK_SerialPort.RtsEnable = true;
        ControllerType = Controller_Type.FC_411; // Default value
    }


    // Controller Type
    public Controller_Type ControllerType
    {
        get
        {
            return Controller;
        }

        set
        {
            Controller = value;
            if (Controller == Controller_Type.FC_911)
            {
                MaxAxis = 2;
                MaxSpeed = 6;
                MinSpeed = 0.000001;
                Resol = 1;

            }
            else if ( Controller == Controller_Type.FC_611)
            {
                MaxAxis = 2;
                MaxSpeed = 30;
                MinSpeed = 0.000005;
                Resol = 5;
            }
            else if (Controller == Controller_Type.FC_511)
            {
                MaxAxis = 2;
                MaxSpeed = 50;
                MinSpeed = 0.00001;
                Resol = 10;
            }
            else if (Controller == Controller_Type.FC_411)
            {
                MaxAxis = 2;
                MaxSpeed = 100;
                MinSpeed = 0.00005;
                Resol = 50;
            }
            else if (Controller == Controller_Type.FC_111)
            {
                MaxAxis = 2;
                MaxSpeed = 100;
                MinSpeed = 0.0001;
                Resol = 100;
            }
        }
    }

    // PortName
    public string PortName
    {
        get
        {
            return SK_SerialPort.PortName;
        }

        set
        {
            SK_SerialPort.PortName = value;
        }
    }

    // Baudrate
    public int Baudrate
    {
        get
        {
            return SK_SerialPort.BaudRate;
        }

        set
        {
            SK_SerialPort.BaudRate = value;
        }
    }

    // Delimiter
    public string Delimiter
    {
        get
        {
            return SK_SerialPort.NewLine;
        }

        set
        {
            SK_SerialPort.NewLine = value;
        }
    }

    // Timeout
    public int Timeout
    {
        get
        {
            return SK_SerialPort.ReadTimeout;
        }

        set
        {
            SK_SerialPort.ReadTimeout = value;
        }
    }

    // PortNames
    public string[] GetPortNames()
    {
        string[] sss;
        sss = SerialPort.GetPortNames();
        return sss;
    }


    // test the conection succes 
    public bool ConnectTest()
    {
        string sss;
        string st;
        
        try
        {

            // デバイスOPEN  device open
            if (SGOpen() == false)
            {
                return false;
            }

            SK_SerialPort.DiscardInBuffer();  // ﾊﾞｯﾌｧｸﾘｱ-->最初にｺﾞﾐが入る  In case of buffer clear->  the level is entered first

            
            if (SGWrite(Get_Mode()) == false)
            {
                return false;
            }


            if (SK_SerialPort.BytesToRead == 0)
            {
                return false;
            }
            // 応答受信 response received
            sss = string.Empty;
            if (SGRead(ref sss) == false)
            {
                return false;
            }

            // 受信文字確認  Check received characters

            st = sss.ToUpper();

            if (st == "LOCAL" || st == "TEACH")
            {
                SGWrite(Remote_Mode());
                return true;
            }
            else if (st == "REMOTE")
            {

                return true;
            }
            else
            {
                return false;
            }
        }
        catch (Exception)
        {
            return false;
        }
    }

    // Port Close
    public bool PortClose()
    {
        bool rtn;
        rtn = SGClose();
        return rtn;
    }

    // Wait to Ready
    public bool Wait_Ready(int Axis)
    {
        string sss;
        string st;
        try
        {
            do
            {
                System.Windows.Forms.Application.DoEvents();
                if (SGWrite(_StatusQ()) == false)
                {
                    return false;
                }

                // 応答受信  response received
                sss = null;
                if (SGRead(ref sss) == false)
                {
                    return false;
                }

                if (!Equals(sss, ""))
                {
                    if (PosText is object)
                    {
                        CurPosLabel.Text = Get_Coordinate_Value(sss, Axis);
                    }

                    st = Get_Stage_Status().ToUpper();
                    if (Equals(st, "R"))
                    {
                        return true;
                    }
                }
            }
            while (true);
        }

        catch (Exception )
        {
            return false;
        }

        
    }
    // Label for current Value
    public Label CurPosLabel
    {
        get
        {
            return PosText;
        }

        set
        {
            PosText = value;
        }
    }

    // 現在位置   Current position
    public int GetPosition( int Axis)
    {
        string sss;
        string pos;

        // Ready確認 Confirmation  
        if (Wait_Ready( Axis) == false)
        {
            return 0;
        }

        // Qコマンド送信  Q command transmission
        if (SGWrite(_StatusQ()) == false)
        {
            return 0;
        }

        sss = null;
        if (SGRead(ref sss) == false)  // 応答受信   Response received
        {
            return 0;
        }

        pos = Get_Coordinate_Value(sss, MaxAxis); // 座標値を得る  Get coordinate values
        return int.Parse(pos);
    }


    // 機械原点復帰  Mechanical origin return
    public bool ReturnOrigin(int Axis)
    {
       
        // Ready確認  Ready confirmation
        if (Wait_Ready( Axis) == false)
        {
            return false;
        }

        // Hｺﾏﾝﾄﾞ送信    Send H command
        if (SGWrite(_ReturnOrigin(Axis)) == false)
        {
            return false;
        }

        // Ready確認  Ready confirmation
        if (Wait_Ready( Axis) == false)
        {
            return false;
        }

        return true;
    }

    // 移動（相対）  Movement (relative)
    public bool MoveRelativity(int Axis, int Vdata)
    {
        

        // Mコマンド送信  Send M command
        if (SGWrite(_MoveRelativity(Axis, Vdata)) == false)
        {
            return false;
        }
     
        // Gコマンド送信   G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }


        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }


    // 移動（相対）  Movement (relative) both axis
    public bool MoveRelativityW(int Axis, int Vdata1, int Vdata2)
    {


        // Mコマンド送信  Send M command
        if (SGWrite(_MoveRelativityW(Axis, Vdata1, Vdata2)) == false)
        {
            return false;
        }

        // Gコマンド送信   G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }


        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }


    // 移動（相対）  Move relativity by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
    public bool MoveRelativityByUnit(int Unit, int Axis, double Vdata1)
    {


        // Mコマンド送信  Send M command
        if (SGWrite(_MoveRelativityByUnit(Unit,Axis, Vdata1)) == false)
        {
            return false;
        }

        // Gコマンド送信   G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }


        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }

    // Move Absolutely 
    public bool MoveAbsolutely(int Axis, int Vdata)
    {

        // Mコマンド送信  Send M command
        if (SGWrite(_MoveAbsolutely(Axis, Vdata)) == false)
        {
            return false;
        }


        // Gコマンド送信   G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }



        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }


    // Move Absolutely Both axis
    public bool MoveAbsolutelyW(int Axis, int Vdata1, int Vdata2)
    {

        // Mコマンド送信  Send M command
        if (SGWrite(_MoveAbsolutelyW(Axis, Vdata1, Vdata2)) == false)
        {
            return false;
        }


        // Gコマンド送信   G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }


        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }

    // 移動（相対）  Move Absolutely by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
    public bool MoveAbsolutelyByUnit(int Unit, int Axis, int Vdata1)
    {


        // Mコマンド送信  Send M command
        if (SGWrite(_MoveAbsolutelyByUnit(Unit, Axis, Vdata1)) == false)
        {
            return false;
        }

        // Gコマンド送信   G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }


        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }

    // 論理原点設定  Logical origin setting
    public bool SetLogicalOrigin(int Axis)
    {

        // Ready確認  Ready confirmation
        if (Wait_Ready( Axis) == false)
        {
            return false;
        }

        // Rコマンド送信    R command transmission
        if (SGWrite(_SetLogicalOrigin(Axis)) == false)
        {
            return false;
        }

        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }

    // 論理原点復帰   Logical origin return       
    public bool ReturnLogicalOrigin(int Axis)
    {
        

        // Ready確認   Ready confirmation
        if (Wait_Ready( Axis) == false)
        {
            return false;
        }

        // Mコマンド送信   Send M command
        if (SGWrite(_ReturnLogicalOrigin(Axis)) == false)
        {
            return false;
        }

       
        // Ready確認   Ready confirmation
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        return true;
    }

    // 減速停止  Deceleration stop
    public bool StopStage(int Axis)
    {

        // Lｺﾏﾝﾄﾞ送信 Send L command
        if (SGWrite(_StopStage(Axis)) == false)
        {
            return false;
        }

        return true;
    }

    // 非常停止   emergency stop
    public bool StopStageEmergency()
    {
        

        // Lｺﾏﾝﾄﾞ送信  Send L command
        if (SGWrite(_StopStageEmergency()) == false)
        {
            return false;
        }

        return true;
    }

    public bool RemoveEmergency()
    {


        // BECｺﾏﾝﾄﾞ送信  Send BEC: command
        if (SGWrite(_RemoveEmergency()) == false)
        {
            return false;
        }

        return true;
    }


    public bool SGOpen()
    {
        try
        {
            if (SK_SerialPort.IsOpen == false)
            {
                SK_SerialPort.Open();
            }

            return true;
        }
        catch (Exception )
        {
            if (Language == true)
            {
                ErrMsg = "デバイスOPEN時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the device open.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }


    // Close
    public bool SGClose()
    {
        try
        {
            if (SK_SerialPort.IsOpen == true)
            {
                SK_SerialPort.Close();
            }

            return true;
        }
        catch (Exception)
        {
            if (Language == true)
            {
                ErrMsg = "デバイスCLOSE時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the device close.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }


    // Write
    public bool SGWrite(string Wcmd)
    {
        try
        {

            

            if (SK_SerialPort.CtsHolding != false)
            {
                if (Language == true)
                {
                    ErrMsg = "送信時にエラーが発生しました。";
                    ErrTitle = "通信エラー";
                }
                else
                {
                    ErrMsg = "Error occurred at the time of the transmission.";
                    ErrTitle = "Communication Error";
                }

                MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            
            if (Wcmd == string.Empty)
            {
                return false;
            }


            SK_SerialPort.WriteLine(Wcmd);
            return true;
        }
        catch (Exception)
        {
            if (Language == true)
            {
                ErrMsg = "送信時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the transmission.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }


    // Read
    public bool SGRead(ref string Rcmd)
    {
        try
        {
            

                Rcmd = SK_SerialPort.ReadLine();
                return true;
            
           
        }
        catch (Exception)
        {
            

            if (Language == true)
            {
                ErrMsg = "受信時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the reception.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }




    // 移動方向（内部関数）Moving direction (internal function)
    private string GetAxisDir(int Value)
    {
        if (Value < 0)
        {
            return "-";
        }
        else
        {
            return "+";
        }
    }
    // 移動方向（内部関数）Moving direction (internal function)
    private string GetAxisDir(double Value)
    {
        if (Value < 0)
        {
            return "-";
        }
        else
        {
            return "+";
        }
    }

    // 移動方向2（内部関数） Moving direction 2 (internal function)
    private string GetAxisDir2(bool Value)
    {
        if (Value == false)
        {
            return "-";
        }
        else
        {
            return "+";
        }
    }


    
   


    // Get Stage Status
    private string inGet_Stage_Status()
    {

        string wkstr;
        wkstr = string.Empty;
        SGWrite(_Status2());
        SGRead(ref wkstr);

        return wkstr; 
    }


    // Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command
    private int Get_Qlen(int AxesNum)
    {
        switch (AxesNum) // 軸数  Number of axes
        {

            case 2:
                {
                    return 27;
                }

            case 1:
                {
                    return 16;
                }

            default:
                {
                    return 0;
                }
        }
    }


    // Return origin
    private string _ReturnOrigin(int Axis)
    {
        if (Axis == 0)  
        {
            return "H:W";
        }
        else if (Axis >= 1 && Axis <= MaxAxis)  
        {
            return "H:" + Axis.ToString();
        }
        else
        {
            return "";
        }
    }

    // Return Logical origin
    private string _ReturnLogicalOrigin(int Axis)
    {
        if (Axis == 0)
        {
            return "Z:W";
        }
        else if (Axis >= 1 && Axis <= MaxAxis)
        {
            return "Z:" + Axis.ToString();
        }
        else
        {
            return "";
        }
    }


    // Move relativity
    private string _MoveRelativity(int Axis, int Value1)
    {
        string WkStr;
        if (Axis == 1)  
        {
            WkStr = "M:1" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
            return WkStr;
        }
        else if (Axis >= 1 && Axis <= MaxAxis)  
        {
            WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
            return WkStr;
        }
        else
        {
            return "";
        }
    }

    // Move relativity both axis
    private string _MoveRelativityW(int Axis, int Value1, int Value2)
    {
        string WkStr;
        if (Axis == 0)  
        {
            WkStr = "M:W" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString() + GetAxisDir(Value2) + "P" + Math.Abs(Value2).ToString();
            return WkStr;
        }
        else
        {
            return "";
        }
    }

    // Move relativity by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
    private string _MoveRelativityByUnit(int Unit,int Axis, double Value1)
    {
        string WkStr;
        if (Unit == 1)
        {
            if (Value1 < Resol)
            {
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";  // Value must be bigger or equal to resolution
            }
            else
            {
                if (Axis == 1)  
                {
                    WkStr = "M:1" + GetAxisDir(Value1) + "N" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else if (Axis >= 1 && Axis <= MaxAxis)  
                {
                    WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "N" + Math.Abs(Value1).ToString();
                    return WkStr;

                }
                else
                {
                    return "";
                }
            }
        }
        else if (Unit == 2)
        {
            
            if (Value1 < Resol / 1000)
            {
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";  // Value must be bigger or equal to resolution
            }
            else
            {
                if (Axis == 1)  
                {
                    WkStr = "M:1" + GetAxisDir(Value1) + "U" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else if (Axis >= 1 && Axis <= MaxAxis)  
                {
                    WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "U" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else
                {
                    return "";
                }
            }

        }
        else if (Unit == 3)
        {
            if (Value1 < Resol / 100000)
            {
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";  // Value must be bigger or equal to resolution
            }
            else
            {
                if (Axis == 1)  
                {
                    WkStr = "M:1" + GetAxisDir(Value1) + "M" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else if (Axis >= 1 && Axis <= MaxAxis)  
                {
                    WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "M" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else
                {
                    return "";
                }
            }

        }
        else if (Unit == 4)
        {
            if (Axis == 1)  
            {
                WkStr = "M:1" + GetAxisDir(Value1) + "D" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  
            {
                WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "D" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
        else
        {
            return "";
        }
    }

    // Move absolutely
    private string _MoveAbsolutely(int Axis, int Value1)
    {
        string WkStr;
        if (Axis == 1)  
        {
            WkStr = "A:1" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
            return WkStr;
        }
        else if (Axis >= 1 && Axis <= MaxAxis)  
        {
            WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
            return WkStr;
        }
        else
        {
            return "";
        }
    }

    // Move absolutely both axis
    private string _MoveAbsolutelyW(int Axis, int Value1, int Value2)
    {
        string WkStr;
        if (Axis == 0)  
        {
            WkStr = "A:W" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString() + GetAxisDir(Value2) + "P" + Math.Abs(Value2).ToString();
            return WkStr;
        }
        else
        {
            return "";
        }
    }

    // Move absolutely by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
    private string _MoveAbsolutelyByUnit(int Unit, int Axis, int Value1)
    {
        string WkStr;
        if (Unit == 1)
        {
            if (Value1 < Resol)
            {
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";  // Value must be bigger or equal to resolution
            }
            else
            {
                if (Axis == 1)  
                {
                    WkStr = "A:1" + GetAxisDir(Value1) + "N" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else if (Axis >= 1 && Axis <= MaxAxis)  
                {
                    WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "N" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else
                {
                    return "";
                }
            }
        }
        else if (Unit == 2)
        {
            if (Value1 < Resol / 1000)
            {
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";  // Value must be bigger or equal to resolution
            }
            else
            {
                if (Axis == 1)  
                {
                    WkStr = "A:1" + GetAxisDir(Value1) + "U" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else if (Axis >= 1 && Axis <= MaxAxis)  
                {
                    WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "U" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else
                {
                    return "";
                }
            }

        }
        else if (Unit == 3)
        {
            if (Value1 < Resol / 100000)
            {
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return "";  // Value must be bigger or equal to resolution
            }
            else
            {
                if (Axis == 1)  
                {
                    WkStr = "A:1" + GetAxisDir(Value1) + "M" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else if (Axis >= 1 && Axis <= MaxAxis)  
                {
                    WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "M" + Math.Abs(Value1).ToString();
                    return WkStr;
                }
                else
                {
                    return "";
                }
            }

        }
        else if (Unit == 4)
        {
            if (Axis == 1)  
            {
                WkStr = "A:1" + GetAxisDir(Value1) + "D" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  
            {
                WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "D" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
        else
        {
            return "";
        }
    }



    // Move with JOG           Speed H (depends Home speed) = 10
    private string _Jog(int Axis, bool Dir_Renamed, int Speed)
    {
        string WkStr;

        if (Speed >= 0 && Speed <= 10)
        {
            if (Axis == 0)  
            {
                if (Speed == 10)
                {
                    WkStr = "JY:W" + " " + GetAxisDir2(Dir_Renamed) + "H";
                    return WkStr;
                }
                else
                {
                    WkStr = "JY:W" + " " + GetAxisDir2(Dir_Renamed) + Speed.ToString();
                    return WkStr;
                }
            }
            else if (Axis == 1 && Axis <= MaxAxis)  
            {
                if (Speed == 10)
                {
                    WkStr = "JY:" + Axis.ToString() + " " + GetAxisDir2(Dir_Renamed) + "H";
                    return WkStr;
                }
                else
                {
                    WkStr = "JY:" + Axis.ToString() + " " + GetAxisDir2(Dir_Renamed) + Speed.ToString();
                    return WkStr;

                }

               
            }
            else
            {
                return "";
            }
        }
        else
        {
            MessageBox.Show(" Speed Value should be between 0 and 10", "Speed Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return "";
        }
    }


    // Drive
    private string _Go()
    {
        return "G";
    }


    // Return logical origin
    private string _SetLogicalOrigin(int Axis)
    {
        if (Axis == 0)  
        {
            return "R:W";
        }
        else if (Axis == 1 && Axis <= MaxAxis)  
        {
            return "R:" + Axis.ToString();
        }
        else
        {
            return "";
        }
    }




    // Emergency stop
    private string _StopStageEmergency()
    {
        return "L:E";
    }

    // remove the emergency 
    private string _RemoveEmergency()
    {

        return "BEC:";
    }


    // Stop
    private string _StopStage(int Axis)
    {
        if (Axis == 0)  
        {
            return "L:1";
        }
        else if (Axis >= 1 && Axis <= MaxAxis)  
        {
            return "L:" + Axis.ToString();
        }
        else
        {
            return "";
        }
    }





    // Speed both axis
    private string _SpeedW(int Axis, double Fast1, double Fast2)
    {
        string WkStr;
        if (Fast1 < MinSpeed && Fast1 > MaxSpeed && Fast2 < MinSpeed && Fast2 > MaxSpeed)
        {
            MessageBox.Show(" Speed set should be more than Min Speed and less than Max Speed ", "Speed Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return "";  // Value must be bigger or equal to resolution
            
        }

        if (Axis == 0)  
        {
            WkStr = "D:W" + "F" + Math.Abs(Fast1).ToString() + "F" + Math.Abs(Fast2).ToString();
            return WkStr;
        }
        else
        {
            return "";
        }
    }
    // Speed only one axis 1 or 2
    private string _Speed(int Axis,  double Fast)
    {
        string WkStr;
        if (Fast < MinSpeed && Fast > MaxSpeed )
        {
            return "";
        }


        if (Axis == 1 && Axis <= MaxAxis)  
        {
            WkStr = "D:" + Axis.ToString() + "F" + Math.Abs(Fast).ToString() ;
            return WkStr;
        }
        else
        {
            return "";
        }
    }

    // Move Jog 

    public bool Jog(int Axis, bool Dir_Renamed, int Speed)
    {
        if (Wait_Ready(Axis) == false)
        {
            return false;
        }

        if  (SGWrite (_Jog(Axis, Dir_Renamed, Speed))== false)
        {
            return false;
        }

        if (Wait_Ready(Axis) == false)
        {
            return false;
        }
        return true;

    }





    // Speed by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree) ()


    private string _SpeedByUnit(int Unit, int Axis, double Fast)
    {
        string WkStr;
        if (Unit == 1)
        {
            if (Fast < MinSpeed && Fast > MaxSpeed)
            {
                return "";
            }
            else
            {
                WkStr = "D:" + Axis.ToString() + "N" + Math.Abs(Fast).ToString();
                return WkStr;
            }
        }
        else if (Unit == 2)
        {
            if (Fast < MinSpeed && Fast > MaxSpeed)
            {
                return "";
            }
            else
            {
                WkStr = "D:" + Axis.ToString() + "U" + Math.Abs(Fast).ToString();
                return WkStr;
            }
        }
        else if (Unit == 3)
        {
            if (Fast < MinSpeed && Fast > MaxSpeed)
            {
                return "";
            }
            else
            {
                WkStr = "D:" + Axis.ToString() + "M" + Math.Abs(Fast).ToString();
                return WkStr;
            }
        }
        else if (Unit == 4)
        {
            if (Fast < MinSpeed && Fast > MaxSpeed)
            {
                return "";
            }
            else
            {
                WkStr = "D:" + Axis.ToString() + "D" + Math.Abs(Fast).ToString();
                return WkStr;
            }
        }
        else
        {

            return "";
        }



    }

    // Speed both axis
    public bool SpeedW(int Axis, double Fast1, double Fast2)
    {
        
        if (SGWrite(_SpeedW(Axis, Fast1, Fast2)) == false)
        {
            return false;
        }

        return true;

    }

    // Speed only one axis 1 or 2
    public bool Speed(int Axis, double Fast)
    {

        if (SGWrite(_Speed(Axis, Fast)) == false)
        {
            return false;
        }

        return true;

    }
    // Speed by unit 

    public bool SpeedByUnit(int Unit , int Axis, double Fast)
    {
    

        if (SGWrite(_SpeedByUnit(Unit, Axis, Fast)) == false)
        {
            return false;
        }

        return true;

    }

    // Get the status "Q"
    private string _StatusQ()
    {
        return "Q:";
    }

    // Get the status 

    public bool StatusQ()
    {

        if (Wait_Ready(0)== false)
        {
            return false;
        }
        if (SGWrite(_StatusQ())==false)
        {
            return false;
        }
        if (Wait_Ready(0) == false)
        {
            return false;
        }
        return true;

    }



    // Get the status "!"
    private string _Status2()
    {
        return "!:";
    }

    // Get the status "!"
    public bool Status2()
    {

        if (Wait_Ready(0) == false)
        {
            return false;
        }
        if (SGWrite(_Status2()) == false)
        {
            return false;
        }
        if (Wait_Ready(0) == false)
        {
            return false;
        }
        return true;

    }



    // Get the status "?"
    private string _Status3(int Axis)
    {
        if (Axis == 0)
            { return "STS:"; }
        else if (Axis == 1 && Axis <= MaxAxis)  // all axis
            { return "STS:" + Axis; }
        else
            { return "Q:"; }

    }

    // Get the coordinate value from the status "Q"
    private string Get_Coordinate_Value(string Qstatus, int Axis )
    {
        int stp;
        //int AxLen;
        string BeforeStr;
        string AfterStr;

        // 取出す先頭  Retrieving head
        switch (Axis)
        {

            case 2:
                {
                    stp = 11;
                    break;
                }

            default:
                {
                    stp = 1;
                    break;
                }
            case 0:
                {
                    stp = 19;
                    break;
                }
        }

        if (stp == 19)
        {
            BeforeStr = Qstatus.Substring(0 , stp);
            AfterStr = BeforeStr.Replace(" ", "");
            return AfterStr;

        }
        else
        {
            BeforeStr = Qstatus.Substring(stp - 1, 10); // -1:1文字目のindexは0のため  -1: Since the index of the first character is 0
            AfterStr = BeforeStr.Replace(" ", "");
            AfterStr = AfterStr.Substring(0, AfterStr.IndexOf(","));
            return AfterStr;
        }

    }


    // Get the status from the status "Q"
    private string Get_Stage_Status()
    {
        return inGet_Stage_Status();
    }


    // get Mode of Controller 
    private string Get_Mode()
    {
        return "MODE?";
    }

    // Remote Mode 
    private string Remote_Mode()
    {
        return "MODE:REMOTE";
    }

    // Get model of controller
    private string Get_Model()
    {

        return "MODEL:";

    }

    public bool Set_Model_Controller()
    {

        string Wkstr;
        Wkstr = string.Empty;
        SGWrite(Get_Model());

        Thread.Sleep(100);
        SGRead(ref Wkstr);

        if (Wkstr == "FC-111")
        {
            ControllerType = Controller_Type.FC_111;
            return true;
        }
        else if (Wkstr == "FC-411")
        {
            ControllerType = Controller_Type.FC_411;
            return true;
        }
        else if (Wkstr == "FC-511")
        {
            ControllerType = Controller_Type.FC_511;
            return true;
        }
        else if (Wkstr == "FC-611")
        {
            ControllerType = Controller_Type.FC_611;
            return true;
        }
        else if (Wkstr == "FC-911")
        {
            ControllerType = Controller_Type.FC_911;
            return true;
        }
        else
        {
            return false;
        }

    }

}