﻿//----------------------------------------------------------------------------------------------
//	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
//	This class can be used by anyone provided that the copyright notice remains intact.
//	Program developed by ABED Toufik, 
//
//	This class helper is used to operate SIGMAKOKI Controllers FC series. 
//	Use the class to build your own program.
//	We can provide support for the other controllers.
//	Contact us : Phone (+ 81-3-5638-8228) or sales@sigma-koki.com.
//----------------------------------------------------------------------------------------------

#include <windows.h>
#include "SKSampleClass.h"


using namespace System::Windows::Forms;
using namespace System::Threading;

SKSampleClass::SKSampleClass()
{
	SK_SerialPort = gcnew SerialPort();
	SK_SerialPort->PortName = L"COM1"; // Default value
	SK_SerialPort->BaudRate = 9600; // Default value
	SK_SerialPort->NewLine = L"\r\n";
	; // Default value
	SK_SerialPort->ReadTimeout = 5000; // Default value
	SK_SerialPort->RtsEnable = true;
	setControllerType(Controller_Type::FC_411); // Default value
}

SKSampleClass::Controller_Type SKSampleClass::getControllerType() 
{
	return Controller;
}

void SKSampleClass::setControllerType(Controller_Type value)
{
	Controller = value;
	if (Controller == Controller_Type::FC_911)
	{
		MaxAxis = 2;
		MaxSpeed = 6;
		MinSpeed = 0.000001;
		Resol = 1;

	}
	else if (Controller == Controller_Type::FC_611)
	{
		MaxAxis = 2;
		MaxSpeed = 30;
		MinSpeed = 0.000005;
		Resol = 5;
	}
	else if (Controller == Controller_Type::FC_511)
	{
		MaxAxis = 2;
		MaxSpeed = 50;
		MinSpeed = 0.00001;
		Resol = 10;
	}
	else if (Controller == Controller_Type::FC_411)
	{
		MaxAxis = 2;
		MaxSpeed = 100;
		MinSpeed = 0.00005;
		Resol = 50;
	}
	else if (Controller == Controller_Type::FC_111)
	{
		MaxAxis = 2;
		MaxSpeed = 100;
		MinSpeed = 0.0001;
		Resol = 100;
	}
}

String^ SKSampleClass::getPortName()
{
	return SK_SerialPort->PortName;
}

void SKSampleClass::setPortName(String^ &value)
{
	SK_SerialPort->PortName = value;
}

int SKSampleClass::getBaudrate() 
{
	return SK_SerialPort->BaudRate;
}

void SKSampleClass::setBaudrate(int value)
{
	SK_SerialPort->BaudRate = value;
}

String^ SKSampleClass::getDelimiter()
{
	return SK_SerialPort->NewLine;
}

void SKSampleClass::setDelimiter(String^ &value)
{
	SK_SerialPort->NewLine = value;
}

int SKSampleClass::getTimeout() 
{
	return SK_SerialPort->ReadTimeout;
}

void SKSampleClass::setTimeout(int value)
{
	SK_SerialPort->ReadTimeout = value;
}

array<Object^>^ SKSampleClass::GetPortNames()
{
	array<Object^>^ sss;
	sss = SK_SerialPort->GetPortNames();
	return sss;
}

bool SKSampleClass::ConnectTest()
{
	String^ sss;
	String^ st;
	
	try
	{

		// デバイスOPEN  device open
		if (SGOpen() == false)
		{
			return false;
		}

		SK_SerialPort->DiscardInBuffer(); // ﾊﾞｯﾌｧｸﾘｱ-->GIP101の場合,最初にｺﾞﾐが入る  In case of buffer clear-> GIP101, the level is entered first

		// Qｺﾏﾝﾄﾞ送信 Send Q command
		if (SGWrite(Get_Mode()) == false)
		{
			return false;
		}

		// 応答受信 response received
		sss = "";
		if (SK_SerialPort->BytesToRead == 0)
		{
			return false;
		}
		if (SGRead(sss) == false)
		{
			return false;
		}

		// 受信文字確認  Check received characters
		
		st = sss->ToUpper();
		
		if ((st == L"LOCAL") || (st == L"TEACH"))
		{
			SGWrite(Remote_Mode());
			return true;
		}
		else if (st == "REMOTE")
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	catch (UnauthorizedAccessException^)
	{
		return false;
	}
}

bool SKSampleClass::PortClose()
{
	bool rtn;
	rtn = SGClose();
	return rtn;
}

// Wait Ready 
bool SKSampleClass::Wait_Ready(int Axis)
{
	String^ sss;
	String^ st;
	try
	{
		do
		{
			System::Windows::Forms::Application::DoEvents();
			if (SGWrite(_StatusQ()) == false)
			{
				return false;
			}

			// 応答受信  response received
			sss = String::Empty;
			if (SGRead(sss) == false)
			{
				return false;
			}

			if ((sss != ""))
			{
				
				getCurPosLabel()->Text = Get_Coordinate_Value(sss, Axis);
				st = (Get_Stage_Status())->ToUpper();

				if ((st == "R"))
				{
					return true;
				}
				
			}
		} while (true);
	}

	catch (UnauthorizedAccessException^)
	{
		return false;
	}


}

// Get Label position
Label ^SKSampleClass::getCurPosLabel()
{
	return PosText;
}
// set Label Position
void SKSampleClass::setCurPosLabel(Label ^value)
{
	PosText = value;
}

// get position 
int SKSampleClass::GetPosition(int Axis)
{
	String^  sss;
	String^  pos;

	// Ready確認 Confirmation  
	if (Wait_Ready(Axis) == false)
	{
		return 0;
	}

	// Qコマンド送信  Q command transmission
	if (SGWrite(_StatusQ()) == false)
	{
		return 0;
	}

	sss = L"";
	if (SGRead(sss) == false) // 応答受信   Response received
	{
		return 0;
	}

	pos = Get_Coordinate_Value(sss, MaxAxis); // 座標値を得る  Get coordinate values
	return System::Int32::Parse(pos);
}

// 機械原点復帰  Mechanical origin return
bool SKSampleClass::ReturnOrigin(int Axis)
{



	// Ready確認 Confirmation  
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}
	// Hｺﾏﾝﾄﾞ送信    Send H command
	if (SGWrite(_ReturnOrigin(Axis)) == false)
	{
		return false;
	}


	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;
}

//Return Origin 

String^ SKSampleClass::_ReturnOrigin(int Axis)
{
	if (Axis == 0) {
		return "H:W";
	
	}
	else if (Axis >= 1&& Axis <= MaxAxis)
	{
		return "H:" + Axis.ToString();
	}
	else
	{
		return "";
	}

}

// Write 
bool SKSampleClass::SGWrite(String^ Wcmd)
{
	try
	{
		if (SK_SerialPort->CtsHolding != false)
		{
			if (Language == true)
			{
				ErrMsg = "送信時にエラーが発生しました。";
				ErrTitle = "通信エラー";
			}
			else
			{
				ErrMsg = "Error occurred at the time of the transmission.";
				ErrTitle = "Communication Error";
			}
			MessageBox::Show( ErrMsg, ErrTitle);
			return false;
		}

		if (Wcmd == String::Empty)
		{
			return false;
		}

		SK_SerialPort->WriteLine(Wcmd);
		return true;

	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "送信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the transmission.";
			ErrTitle = "Communication Error";
		}
		MessageBox::Show(ErrMsg, ErrTitle);
		return false;

	}
}

//read 
bool SKSampleClass::SGRead( String^ &Rcmd)
{
	try
	{
		
			Rcmd = SK_SerialPort->ReadLine();
			return true;
	
	

	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "受信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the reception.";
			ErrTitle = "Communication Error";
		}

		MessageBox::Show(ErrMsg, ErrTitle);
		return false;
	}


}

//status Q

String^ SKSampleClass::_StatusQ()
{
	return "Q:";
}

//Open

bool SKSampleClass::SGOpen()
{
	try
	{
		if (SK_SerialPort->IsOpen == false) 
		{

			SK_SerialPort->Open();
		}
		return true;

	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "受信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the reception.";
			ErrTitle = "Communication Error";
		}

		MessageBox::Show(ErrMsg, ErrTitle);
		return false;
	}
}
//Close 

bool SKSampleClass::SGClose()
{
	try
	{
		if (SK_SerialPort->IsOpen == true) 
		{

			SK_SerialPort->Close();
		}

		return true;
	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "受信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the reception.";
			ErrTitle = "Communication Error";
		}

		MessageBox::Show(ErrMsg, ErrTitle);
		return false;

	}
}

// Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command

int SKSampleClass::Get_Qlen(int AxisNum)
{
	switch (AxisNum)
	{
		case 2:
		{
			return 27;
		}
		case 1:
		{
			return 16;
		}
		default:
		{
			return 0;
		}
	}



}

// Qｺﾏﾝﾄﾞからｽﾃｰﾀｽを取出す  Extract status from Q-command
String^ SKSampleClass::inGet_Stage_Status()
{

	String ^ wkstr;
	wkstr = String::Empty;
	SGWrite(_Status2());
	SGRead(wkstr);


	return wkstr; 

}

// Get the status from the status "Q"
String ^ SKSampleClass::Get_Stage_Status()
{

	return inGet_Stage_Status();

}



// Get the coordinate value from the status "Q"
String^  SKSampleClass::Get_Coordinate_Value(String^  Qstatus, int Axis)
{

	int stp;
	String^ BeforeStr;
	String^ AfterStr;

	// 取出す先頭  Retrieving head
	switch (Axis)
	{
	case 2:
	{
		stp = 11;
		break;
	}
	case 0:
	{
		stp = 19;
		break;
	}
	default:
	{
		stp = 1;
		break;
	}
	}

	if (stp == 19)
	{
		BeforeStr = Qstatus->Substring(0, stp);
		AfterStr = BeforeStr->Replace(" ", "");
		return AfterStr;
	}
	else
	{
		BeforeStr = Qstatus->Substring(stp - 1, 10); // -1:1文字目のindexは0のため  -1: Since the index of the first character is 0
		AfterStr = BeforeStr->Replace(" ", "");
		AfterStr = AfterStr->Substring(0, AfterStr->IndexOf(","));
		return AfterStr;
	}

}

// Move relative
String^ SKSampleClass::_MoveRelativity(int Axis, int Value1)
{
	String^ WkStr;
	if (Axis == 1)  
	{
		WkStr = "M:1" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
		return WkStr;
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  
	{
		WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}


}


// Move Relativity by Unit set(1: Nanometer, 2 : Micrometer, 3 : Milimeter, 4 : Degree)
String^ SKSampleClass::_MoveRelativityByUnit(int Unit, int Axis, int Value1)
{
	String^ WkStr;
	if (Unit == 1)
	{
		if (Value1 < Resol)
		{
			MessageBox::Show(" Value should be equal or bigger than resolution of controller", "Resolution Error");
			return "";  // Value must be bigger or equal to resolution
		}
		else
		{
			if (Axis == 1)  
			{
				WkStr = "M:1" + GetAxisDir(Value1) + "N" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else if (Axis >= 1 && Axis <= MaxAxis)  
			{
				WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "N" + Math::Abs(Value1).ToString();
				return WkStr;

			}
			else
			{
				return "";
			}
		}
	}
	else if (Unit == 2)
	{

		if (Value1 < Resol / 1000)
		{
			MessageBox::Show(" Value should be equal or bigger than resolution of controller", "Resolution Error");
			return "";  // Value must be bigger or equal to resolution
		}
		else
		{
			if (Axis == 1)  
			{
				WkStr = "M:1" + GetAxisDir(Value1) + "U" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else if (Axis >= 1 && Axis <= MaxAxis)  
			{
				WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "U" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else
			{
				return "";
			}
		}

	}
	else if (Unit == 3)
	{
		if (Value1 < Resol / 100000)
		{
			MessageBox::Show(" Value should be equal or bigger than resolution of controller", "Resolution Error");
			return "";  // Value must be bigger or equal to resolution
		}
		else
		{
			if (Axis == 1) 
			{
				WkStr = "M:1" + GetAxisDir(Value1) + "M" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else if (Axis >= 1 && Axis <= MaxAxis)  
			{
				WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "M" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else
			{
				return "";
			}
		}

	}
	else if (Unit == 4)
	{
		if (Axis == 1)  
		{
			WkStr = "M:1" + GetAxisDir(Value1) + "D" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else if (Axis >= 1 && Axis <= MaxAxis)  
		{
			WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "D" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else
		{
			return "";
		}
	}
	else
	{
		return "";
	}


}

// Move Relativity both axis
String^ SKSampleClass::_MoveRelativityW(int Axis, int Value1, int Value2)
{
	String^ WkStr;
	if (Axis == 0)  
	{
		WkStr = "M:W" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString() + GetAxisDir(Value2) + "P" + Math::Abs(Value2).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}
}

// move absolute
String^ SKSampleClass::_MoveAbsolutely(int Axis, int Value1)
{
	String^ WkStr;
	if (Axis == 1)  
	{
		WkStr = "A:1" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
		return WkStr;
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  
	{
		WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}


}

// Move absolutely by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
String^ SKSampleClass::_MoveAbsolutelyByUnit(int Unit, int Axis, int Value1)
{
	String^ WkStr;
	if (Unit == 1)
	{
		if (Value1 < Resol)
		{
			MessageBox::Show(" Value should be equal or bigger than resolution of controller", "Resolution Error");
			return "";  // Value must be bigger or equal to resolution
		}
		else
		{
			if (Axis == 1)  
			{
				WkStr = "A:1" + GetAxisDir(Value1) + "N" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else if (Axis >= 1 && Axis <= MaxAxis)  
			{
				WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "N" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else
			{
				return "";
			}
		}
	}
	else if (Unit == 2)
	{
		if (Value1 < Resol / 1000)
		{
			MessageBox::Show(" Value should be equal or bigger than resolution of controller", "Resolution Error");
			return "";  // Value must be bigger or equal to resolution
		}
		else
		{
			if (Axis == 1)  
			{
				WkStr = "A:1" + GetAxisDir(Value1) + "U" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else if (Axis >= 1 && Axis <= MaxAxis)  
			{
				WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "U" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else
			{
				return "";
			}
		}

	}
	else if (Unit == 3)
	{
		if (Value1 < Resol / 100000)
		{
			MessageBox::Show(" Value should be equal or bigger than resolution of controller", "Resolution Error");
			return "";  // Value must be bigger or equal to resolution
		}
		else
		{
			if (Axis == 1)  
			{
				WkStr = "A:1" + GetAxisDir(Value1) + "M" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else if (Axis >= 1 && Axis <= MaxAxis)  
			{
				WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "M" + Math::Abs(Value1).ToString();
				return WkStr;
			}
			else
			{
				return "";
			}
		}

	}
	else if (Unit == 4)
	{
		if (Axis == 1)  
		{
			WkStr = "A:1" + GetAxisDir(Value1) + "D" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else if (Axis >= 1 && Axis <= MaxAxis)  
		{
			WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "D" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else
		{
			return "";
		}
	}
	else
	{
		return "";
	}

}
// Move absolutely both axis
String^ SKSampleClass::_MoveAbsolutelyW(int Axis, int Value1, int Value2)
{
	String^ WkStr;
	if (Axis == 0)  
	{
		WkStr = "A:W" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString() + GetAxisDir(Value2) + "P" + Math::Abs(Value2).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}

}
//get direction 
String^ SKSampleClass::GetAxisDir(int Value)
{
	if (Value <0)
	{
		return "-";
	}
	else
	{
		return "+";
	}

}
//get direction 
String^ SKSampleClass::GetAxisDir(double Value)
{
	if (Value <0)
	{
		return "-";
	}
	else
	{
		return "+";
	}

}

//get Bool direction
String^ SKSampleClass::GetAxisDir2(bool Value)
{
	if (Value == false)
	{
		return "-";
	}
	else
	{
		return "+";
	}
}

// Jog command 
String^ SKSampleClass::_Jog(int Axis, bool Dir_Renamed, int Speed)
{
	String^ WkStr;
	if (Speed >= 0 && Speed <= 10)
	{
		if (Axis == 0)  
		{
			if (Speed == 10)
			{
				WkStr = "JY:W" + " " + GetAxisDir2(Dir_Renamed) + "H";
				return WkStr;
			}
			else
			{
				WkStr = "JY:W" + " " + GetAxisDir2(Dir_Renamed) + Speed.ToString();
				return WkStr;
			}
		}
		else if (Axis == 1 && Axis <= MaxAxis)  
		{
			if (Speed == 10)
			{
				WkStr = "JY:" + Axis.ToString() + " " + GetAxisDir2(Dir_Renamed) + "H";
				return WkStr;
			}
			else
			{
				WkStr = "JY:" + Axis.ToString() + " " + GetAxisDir2(Dir_Renamed) + Speed.ToString();
				return WkStr;

			}


		}
		else
		{
			return "";
		}
	}
	else
	{
		MessageBox::Show(" Speed Value should be between 0 and 10", "Speed Error");
		return "";
	}

}

// Go command
String^ SKSampleClass::_Go()
{

	return "G";
}

// Set Logical Origin
String^ SKSampleClass::_SetLogicalOrigin(int Axis)
{
	if (Axis == 0)  
	{
		return "R:W";
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  
	{
		return "R:" + Axis.ToString();
	}
	else
	{
		return "";
	}

}

// return Logical Origin
String^ SKSampleClass::_ReturnLogicalOrigin(int Axis)
{
	if (Axis == 0)  
	{
		return "Z:W";
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  
	{
		return "Z:" + Axis.ToString();
	}
	else
	{
		return "";
	}

}



// Emergency Stop
String^ SKSampleClass::_StopStageEmergency()
{
	return "L:E";

}

// Stop stages by axis 
String^ SKSampleClass::_StopStage(int Axis)
{
	if (Axis == 0)  
	{
		return "L:W";
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  
	{
		return "L:" + Axis.ToString();
	}
	else
	{
		return "";
	}

}

// Set Speed
String^ SKSampleClass::_Speed(int Axis, double Fast)
{
	String^ WkStr;
	if (Fast < MinSpeed && Fast > MaxSpeed)
	{
		return "";
	}


	if (Axis == 1 && Axis <= MaxAxis)  
	{
		WkStr = "D:" + Axis.ToString() + "F" + Math::Abs(Fast).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}
}

// Speed by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
String^ SKSampleClass::_SpeedByUnit(int Unit, int Axis, double Fast)
{
	String^ WkStr;
	if (Unit == 1)
	{
		if (Fast < MinSpeed && Fast > MaxSpeed)
		{
			return "";
		}
		else
		{
			WkStr = "D:" + Axis.ToString() + "N" + Math::Abs(Fast).ToString();
			return WkStr;
		}
	}
	else if (Unit == 2)
	{
		if (Fast < MinSpeed && Fast > MaxSpeed)
		{
			return "";
		}
		else
		{
			WkStr = "D:" + Axis.ToString() + "U" + Math::Abs(Fast).ToString();
			return WkStr;
		}
	}
	else if (Unit == 3)
	{
		if (Fast < MinSpeed && Fast > MaxSpeed)
		{
			return "";
		}
		else
		{
			WkStr = "D:" + Axis.ToString() + "M" + Math::Abs(Fast).ToString();
			return WkStr;
		}
	}
	else if (Unit == 4)
	{
		if (Fast < MinSpeed && Fast > MaxSpeed)
		{
			return "";
		}
		else
		{
			WkStr = "D:" + Axis.ToString() + "D" + Math::Abs(Fast).ToString();
			return WkStr;
		}
	}
	else
	{

		return "";
	}
}

// Speed both axis
String^ SKSampleClass::_SpeedW(int Axis, double Fast1, double Fast2)
{
	String^ WkStr;
	if (Fast1 < MinSpeed && Fast1 > MaxSpeed && Fast2 < MinSpeed && Fast2 > MaxSpeed)
	{
		MessageBox::Show(" Speed set should be more than Min Speed and less than Max Speed ", "Speed Error");
		return "";  // Value must be bigger or equal to resolution

	}

	if (Axis == 0)  
	{
		WkStr = "D:W" + "F" + Math::Abs(Fast1).ToString() + "F" + Math::Abs(Fast2).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}

}


// Get the status "!"
String^ SKSampleClass::_Status2()
{
	return "!:";

}


String^ SKSampleClass::_Status3(int Axis)
{
	if (Axis == 0)
	{
		return "STS:";
	}
	else if (Axis == 1 && Axis <= MaxAxis)  
	{
		return "STS:" + Axis;
	}
	else
	{
		return "Q:";
	}

}

// Get mode of contrtoller
String^ SKSampleClass::Get_Mode()
{
	return "MODE?";
}

// Remove Emergency
String^ SKSampleClass::_RemoveEmergency()
{
	return "BEC:";
}

// Set Remote Mode
String^ SKSampleClass::Remote_Mode()
{
	return "MODE:REMOTE";

}

// Get Model Of Controller

String^ SKSampleClass::Get_Model()
{
	return "MODEL:";

}


// Remove Emergency 
bool SKSampleClass::RemoveEmergency()
{
	// BECｺﾏﾝﾄﾞ送信  Send BEC : command
		if (SGWrite(_RemoveEmergency()) == false)
		{
			return false;
		}

	return true;

}

// Set Controller model

bool SKSampleClass::Set_Model_Controller()
{
	String^ Wkstr;
	Wkstr = String::Empty;
	SGWrite(Get_Model());

	
	SGRead(Wkstr);

	if (Wkstr == "FC-111")
	{
		setControllerType(Controller_Type::FC_111);
		return true;
	}
	else if (Wkstr == "FC-411")
	{
		setControllerType(Controller_Type::FC_411);
		return true;
	}
	else if (Wkstr == "FC-511")
	{
		setControllerType(Controller_Type::FC_511);
		return true;
	}
	else if (Wkstr == "FC-611")
	{
		setControllerType(Controller_Type::FC_611);
		return true;
	}
	else if (Wkstr == "FC-911")
	{
		setControllerType(Controller_Type::FC_911);
		return true;
	}
	else
	{
		return false;
	}
}


// 移動（相対）  Movement (relative)
bool SKSampleClass::MoveRelativity(int Axis, int Vdata)
{
	// Mコマンド送信  Send M command
	if (SGWrite(_MoveRelativity(Axis, Vdata)) == false)
	{
		return false;
	}

	// Gコマンド送信   G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}


	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}

// 移動（相対）  Move relativity by Unit set(1: Nanometer, 2 : Micrometer, 3 : Milimeter, 4 : Degree)
bool SKSampleClass::MoveRelativityByUnit(int Unit, int Axis, double Vdata1)
{
	// Mコマンド送信  Send M command
		if (SGWrite(_MoveRelativityByUnit(Unit, Axis, Vdata1)) == false)
		{
			return false;
		}

	// Gコマンド送信   G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}


	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}

// Move Relativity both axis
bool SKSampleClass::MoveRelativityW(int Axis, int Vdata1, int Vdata2)
{
	// Mコマンド送信  Send M command
	if (SGWrite(_MoveRelativityW(Axis, Vdata1, Vdata2)) == false)
	{
		return false;
	}

	// Gコマンド送信   G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}


	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}

bool SKSampleClass::MoveAbsolutely(int Axis, int Vdata)
{
	// Mコマンド送信  Send M command
	if (SGWrite(_MoveAbsolutely(Axis, Vdata)) == false)
	{
		return false;
	}


	// Gコマンド送信   G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}



	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}
// 移動（相対）  Move Absolutely by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
bool  SKSampleClass::MoveAbsolutelyByUnit(int Unit, int Axis, int Vdata1)
{
	// Mコマンド送信  Send M command
	if (SGWrite(_MoveAbsolutelyByUnit(Unit, Axis, Vdata1)) == false)
	{
		return false;
	}

	// Gコマンド送信   G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}


	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;
}

// Move absolutely both axis 
bool SKSampleClass::MoveAbsolutelyW(int Axis, int Vdata1, int Vdata2)
{
	// Mコマンド送信  Send M command
	if (SGWrite(_MoveAbsolutelyW(Axis, Vdata1, Vdata2)) == false)
	{
		return false;
	}


	// Gコマンド送信   G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}


	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}

// 論理原点設定  Logical origin setting
bool SKSampleClass::SetLogicalOrigin(int Axis)
{

	// Ready確認  Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	// Rコマンド送信    R command transmission
	if (SGWrite(_SetLogicalOrigin(Axis)) == false)
	{
		return false;
	}


	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}

//論理原点復帰   Logical origin return
bool SKSampleClass::ReturnLogicalOrigin(int Axis)
{

	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	// Mコマンド送信   Send M command
	if (SGWrite(_ReturnLogicalOrigin(Axis)) == false)
	{
		return false;
	}



	// Ready確認   Ready confirmation
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	return true;

}

// 減速停止  Deceleration stop
bool SKSampleClass::StopStage(int Axis)
{
	
	// Lｺﾏﾝﾄﾞ送信 Send L command
	if (SGWrite(_StopStage(Axis)) == false)
	{
		return false;
	}



	return true;
}
// 非常停止   emergency stop
bool SKSampleClass::StopStageEmergency()
{


	// Lｺﾏﾝﾄﾞ送信  Send L command
	if (SGWrite(_StopStageEmergency()) == false)
	{
		return false;
	}

	

	return true;

}

// Move Jog 

bool SKSampleClass::Jog(int Axis, bool Dir_Renamed, int Speed)
{
	if (Wait_Ready(Axis) == false)
	{
		return false;
	}

	if (SGWrite(_Jog(Axis, Dir_Renamed, Speed)) == false)
	{
		return false;
	}

	if (Wait_Ready(Axis) == false)
	{
		return false;
	}
	return true;

}

// Set Speed
bool SKSampleClass::Speed(int Axis, int Fast)
{
	

	if (SGWrite(_Speed(Axis, Fast)) == false)
	{
		return false;
	}

	return true;
	

}

// Speed by Unit Set 
bool SKSampleClass::SpeedByUnit(int Unit, int Axis, double Fast)
{
	if (SGWrite(_SpeedByUnit(Unit, Axis, Fast)) == false)
	{
		return false;
	}

	return true;

}

// Speed both Axis 
bool SKSampleClass::SpeedW(int Axis, double Fast1, double Fast2)
{
	if (SGWrite(_SpeedW(Axis, Fast1, Fast2)) == false)
	{
		return false;
	}

	return true;

}

//　Status 2 "!"
bool SKSampleClass::Status2()
{
	if (Wait_Ready(0) == false)
	{
		return false;
	}
	if (SGWrite(_Status2()) == false)
	{
		return false;
	}
	if (Wait_Ready(0) == false)
	{
		return false;
	}
	return true;

}