﻿#pragma once
//----------------------------------------------------------------------------------------------
//	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
//	This class can be used by anyone provided that the copyright notice remains intact.
//	Program developed by ABED Toufik, checked and confirmed by K.Nobatake.
//
//	This class helper is used to operate SIGMAKOKI Controllers (GSC01, GIP-101B, SHOT302/304GS). 
//	Use the class to build your own program.
//	We can provide support for the other controllers.
//	Contact us : Phone (+ 81-3-5638-8228) or sales@sigma-koki.com.
//----------------------------------------------------------------------------------------------

#include <string>
#include <vector>
#include <cmath>
#include <stdexcept>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Windows::Forms;
using namespace System::IO::Ports;

public ref class SKSampleClass
{
public:
	enum class Controller_Type
	{
		FC_911,
		FC_611,
		FC_511,
		FC_411,
		FC_111
	};

public:
	SerialPort^ SK_SerialPort;
private:
	Controller_Type Controller = static_cast<Controller_Type>(0);
	int MaxAxis = 0;
	double Resol;      // Resolution of controller 
	double MaxSpeed;   // Max speed of controller, note that Max speed value of stages may differs than the Controller
	double MinSpeed;   // Min speed of controller, note that Min speed value of stages may differs than the Controller
	String^ ErrMsg;
	String^ ErrTitle;
	Label^ PosText;
public:
	bool Language = false;

	virtual ~SKSampleClass()
	{
		delete SK_SerialPort;
		delete PosText;
	}

	SKSampleClass();


	// Controller Type
	Controller_Type getControllerType() ;

	void setControllerType(Controller_Type value);

	// PortName
	String ^ getPortName() ;

	void setPortName(String^ &value);

	// Baudrate
	int getBaudrate() ;

	void setBaudrate(int value);

	// Delimiter
	String ^ getDelimiter();

	void setDelimiter(String^ &value);

	// Timeout
	int getTimeout() ;

	void setTimeout(int value);

	// PortNames
	array<Object^>^ GetPortNames();


	// test the conection succes 
	bool ConnectTest();

	bool PortClose();

	// Wait to Ready
	bool Wait_Ready(int Axis);

	// Get Label postion
	Label ^getCurPosLabel() ;

	// set Label Position 
	void setCurPosLabel(Label ^value);

	// 現在位置   Current position
	int GetPosition(int Axis);


	// 機械原点復帰  Mechanical origin return
	bool ReturnOrigin(int Axis);

	// Set Speed
	bool Speed(int Axis, int Fast);

	// Set Speed by Unit Set 
	bool SpeedByUnit(int Unit, int Axis, double Fast);

	// Speed both Axis 
	bool SpeedW(int Axis, double Fast1, double Fast2);

	//　Status 2 "!"
	bool Status2();

	// 移動（相対）  Movement (relative)
	bool MoveRelativity(int Axis, int Vdata);

	// 移動（相対）  Movement (relative)
	bool MoveAbsolutely(int Axis, int Vdata);
//
	// 論理原点設定  Logical origin setting
	bool SetLogicalOrigin(int Axis);
//
	 //論理原点復帰   Logical origin return
	bool ReturnLogicalOrigin(int Axis);
//
	// 減速停止  Deceleration stop
	bool StopStage(int Axis);
//
	// 非常停止   emergency stop
	bool StopStageEmergency();
//
	
//
	bool SGOpen();
//
//
//	// Close
	bool SGClose();
//
//
	// Write
	bool SGWrite( String^ Wcmd);
//
//
	// Read
	bool SGRead(String^ &Rcmd);
//
//
//
//
	// 移動方向（内部関数）Moving direction (internal function)
private:
	String^ GetAxisDir(int Value);
	String^ GetAxisDir(double Value);

	// 移動方向2（内部関数） Moving direction 2 (internal function)
	String^ GetAxisDir2(bool Value);

//
//
	// Qｺﾏﾝﾄﾞからｽﾃｰﾀｽを取出す  Extract status from Q-command
	String^ inGet_Stage_Status();
//
//
	// Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command
	int Get_Qlen(int AxesNum);
//
//
	// Return origin
	String^ _ReturnOrigin(int Axis);
//
//
	// Move relativity
	String^ _MoveRelativity(int Axis, int Value1);
//

	// Move Relativity by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
	String^ _MoveRelativityByUnit(int Unit, int Axis, int Value1);

	// Move Relativity both axis
	String^ _MoveRelativityW(int Axis, int Value1, int Value2);

	// Move absolutely
	String^ _MoveAbsolutely(int Axis, int Value1);
//
	// Move absolutely by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
	String^ _MoveAbsolutelyByUnit(int Unit, int Axis, int Value1);
//
	// Move absolutely both axis
	String^ _MoveAbsolutelyW( int Axis, int Value1, int Value2);

	// Move with JOG
	String^ _Jog(int Axis, bool Dir_Renamed, int Speed);

//
	// Drive
	String^ _Go();
//
//
	// Return logical origin
	String^ _SetLogicalOrigin(int Axis);


	// Return logical origin
	String^ _ReturnLogicalOrigin(int Axis);
//
	// Emergency stop
	String^ _StopStageEmergency();
//
	// Remove Emergency
	String^ _RemoveEmergency();
//
	// Stop
	String^ _StopStage(int Axis);
//
//
	// Speed
	String^ _Speed(int Axis, double Fast);
//
//
	// Speed by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
	String^ _SpeedByUnit(int Unit, int Axis, double Fast);

	// Speed both axis
	String^ _SpeedW(int Axis, double Fast1, double Fast2);

	// Get the status "Q"
	String ^ _StatusQ();
//
//
	// Get the status "!"
	String^ _Status2();

//
	// Get the status "?"
	String^ _Status3(int Axis);
public:
	// Move Jog 
	bool Jog(int Axis, bool Dir_Renamed, int Speed);
	// Move Absolutely by Unit set (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
	bool MoveAbsolutelyByUnit(int Unit, int Axis, int Vdata1);
	// Move absolutely both axis 
	bool MoveAbsolutelyW(int Axis, int Vdata1, int Vdata2);
	// 移動（相対）  Move relativity by Unit set(1: Nanometer, 2 : Micrometer, 3 : Milimeter, 4 : Degree)
	bool MoveRelativityByUnit(int Unit, int Axis, double Vdata1);
	// Move Relativity both axis
	bool MoveRelativityW(int Axis, int Vdata1, int Vdata2);
	// Remove Emergency 
	bool RemoveEmergency();

	// Set Model controller
	bool Set_Model_Controller();

	// Get the coordinate value from the status "Q"
private:
	String^  Get_Coordinate_Value(String^  Qstatus, int Axis );

	// Get the status from the status "Q"
	String^ Get_Stage_Status();

	// get mode of contrtoller
	String^ Get_Mode();

	// Set Remote Mode
	String^ Remote_Mode();

	// Get Model Of Controller

	String^ Get_Model();


};
