﻿'-----------------------------------------------------------------------------------------------
'	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
'	This class can be used by anyone provided that the copyright notice remains intact.
'	Program developed by ABED Toufik.
'
'	is class helper is used to operate SIGMAKOKI Controllers FC series.
'	Use the class to build your own program.
'	We can provide support for the other controllers.
'	Contact us : Phone(+81 - 3 - 5638 - 8228) Or sales@sigma-koki.com.
'-----------------------------------------------------------------------------------------------

Imports System.IO.Ports


Public Class SKSampleClass
    Public Enum Controller_Type
        FC_911
        FC_611
        FC_511
        FC_411
        FC_111
    End Enum

    Public SK_SerialPort As SerialPort
    Dim Controller As Controller_Type
    Dim MaxAxis As Integer
    Dim Resol As Double    'Resolution of controller 
    Dim MaxSpeed As Double    'Max speed of controller, note that Max speed value of stages may differs than the Controller
    Dim MinSpeed As Double    'Min speed of controller, note that Max speed value of stages may differs than the Controller
    Dim ErrMsg As String
    Dim ErrTitle As String
    Dim PosText As Label
    Public Language As Boolean


    Public Sub New()
        SK_SerialPort = New SerialPort
        SK_SerialPort.PortName = "COM1"     'Default value
        SK_SerialPort.BaudRate = 9600       'Default value
        SK_SerialPort.NewLine = vbCrLf      'Default value
        SK_SerialPort.ReadTimeout = 5000    'Default value
        SK_SerialPort.RtsEnable = True
        ControllerType = Controller_Type.FC_411 'Default value
    End Sub


    'Controller Type
    Public Property ControllerType As Controller_Type
        Get
            Return Controller
        End Get
        Set(value As Controller_Type)
            Controller = value
            If Controller = Controller_Type.FC_911 Then
                MaxAxis = 2
                MaxSpeed = 6
                MinSpeed = 0.000001
                Resol = 1
            ElseIf Controller = Controller_Type.FC_611 Then
                MaxAxis = 2
                MaxSpeed = 30
                MinSpeed = 0.000005
                Resol = 5
            ElseIf Controller = Controller_Type.FC_511 Then
                MaxAxis = 2
                MaxSpeed = 50
                MinSpeed = 0.00001
                Resol = 10
            ElseIf Controller = Controller_Type.FC_411 Then
                MaxAxis = 2
                MaxSpeed = 100
                MinSpeed = 0.00005
                Resol = 50
            ElseIf Controller = Controller_Type.FC_111 Then
                MaxAxis = 2
                MaxSpeed = 100
                MinSpeed = 0.0001
                Resol = 100

            End If
        End Set
    End Property

    'PortName
    Public Property PortName As String
        Get
            Return SK_SerialPort.PortName
        End Get
        Set(value As String)
            SK_SerialPort.PortName = value
        End Set
    End Property

    'Baudrate
    Public Property Baudrate As Integer
        Get
            Return SK_SerialPort.BaudRate
        End Get
        Set(value As Integer)
            SK_SerialPort.BaudRate = value
        End Set
    End Property

    'Delimiter
    Public Property Delimiter As String
        Get
            Return SK_SerialPort.NewLine
        End Get
        Set(value As String)
            SK_SerialPort.NewLine = value
        End Set
    End Property

    'Timeout
    Public Property Timeout As Integer
        Get
            Return SK_SerialPort.ReadTimeout
        End Get
        Set(value As Integer)
            SK_SerialPort.ReadTimeout = value
        End Set
    End Property

    'PortNames
    Public Function GetPortNames() As String()
        Dim sss As String()
        sss = SerialPort.GetPortNames()
        Return sss
    End Function


    'test the conection succes 
    Public Function ConnectTest() As Boolean
        Dim sss As String
        Dim st As String


        Try

            'デバイスOPEN  device open
            If SGOpen() = False Then
                Return False
            End If

            SK_SerialPort.DiscardInBuffer()

            If SGWrite(Get_Mode()) = False Then
                Return False
            End If

            If SK_SerialPort.BytesToRead = 0 Then
                Return False
            End If

            '応答受信 response received
            sss = String.Empty

            If SGRead(sss) = False Then
                Return False
            End If

            '受信文字確認  Check received characters

            st = sss.ToUpper()

            If Equals(st, "LOCAL") OrElse Equals(st, "TEACH") Then
                SGWrite(Remote_Mode())
                Return True
            ElseIf Equals(st, "REMOTE") Then
                Return True
            Else
                Return False
            End If

        Catch __unusedException1__ As Exception
            Return False
        End Try
    End Function

    Public Function PortClose() As Boolean
        Dim rtn As Boolean
        rtn = SGClose()
        Return rtn
    End Function

    'Wait to Ready
    Public Function Wait_Ready(ByVal Axis As Integer) As Boolean
        Dim sss As String
        Dim st As String

        Try

            Do
                System.Windows.Forms.Application.DoEvents()

                If SGWrite(_StatusQ()) = False Then
                    Return False
                End If

                '応答受信  response received
                sss = Nothing

                If SGRead(sss) = False Then
                    Return False
                End If

                If Not Equals(sss, "") Then

                    If Not PosText Is Nothing Then
                        CurPosLabel.Text = Get_Coordinate_Value(sss, Axis)
                    End If

                    st = Get_Stage_Status().ToUpper()

                    If Equals(st, "R") Then
                        Return True
                    End If
                End If
            Loop While True

        Catch __unusedException1__ As Exception
            Return False
        End Try
        Return True
    End Function


    Public Property CurPosLabel As Label
        Get
            Return PosText
        End Get
        Set(ByVal value As Label)
            PosText = value
        End Set
    End Property

    ' Get current position 
    Public Function GetPosition(ByVal Axis As Integer) As Integer
        Dim sss As String
        Dim pos As String

        If Wait_Ready(Axis) = False Then
            Return 0
        End If

        If SGWrite(_StatusQ()) = False Then
            Return 0
        End If

        sss = Nothing

        If SGRead(sss) = False Then
            Return 0
        End If

        pos = Get_Coordinate_Value(sss, MaxAxis)
        Return Integer.Parse(pos)
    End Function

    ' Return Origin 
    Public Function ReturnOrigin(ByVal Axis As Integer) As Boolean
        If Wait_Ready(Axis) = False Then
            Return False
        End If

        If SGWrite(_ReturnOrigin(Axis)) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Move relative 
    Public Function MoveRelativity(ByVal Axis As Integer, ByVal Vdata As Integer) As Boolean
        If SGWrite(_MoveRelativity(Axis, Vdata)) = False Then
            Return False
        End If

        If SGWrite(_Go()) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Move relative both axis
    Public Function MoveRelativityW(ByVal Axis As Integer, ByVal Vdata1 As Integer, ByVal Vdata2 As Integer) As Boolean
        If SGWrite(_MoveRelativityW(Axis, Vdata1, Vdata2)) = False Then
            Return False
        End If

        If SGWrite(_Go()) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Move relative by unit (UNIT =1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
    Public Function MoveRelativityByUnit(ByVal Unit As Integer, ByVal Axis As Integer, ByVal Vdata1 As Double) As Boolean
        If SGWrite(_MoveRelativityByUnit(Unit, Axis, Vdata1)) = False Then
            Return False
        End If

        If SGWrite(_Go()) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' move absolutely 
    Public Function MoveAbsolutely(ByVal Axis As Integer, ByVal Vdata As Integer) As Boolean
        If SGWrite(_MoveAbsolutely(Axis, Vdata)) = False Then
            Return False
        End If

        If SGWrite(_Go()) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Move absolute for both axis 
    Public Function MoveAbsolutelyW(ByVal Axis As Integer, ByVal Vdata1 As Integer, ByVal Vdata2 As Integer) As Boolean
        If SGWrite(_MoveAbsolutelyW(Axis, Vdata1, Vdata2)) = False Then
            Return False
        End If

        If SGWrite(_Go()) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Move absolute by unit (UNIT 1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree)
    Public Function MoveAbsolutelyByUnit(ByVal Unit As Integer, ByVal Axis As Integer, ByVal Vdata1 As Integer) As Boolean
        If SGWrite(_MoveAbsolutelyByUnit(Unit, Axis, Vdata1)) = False Then
            Return False
        End If

        If SGWrite(_Go()) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function

    ' Set position to 0
    Public Function SetLogicalOrigin(ByVal Axis As Integer) As Boolean

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        If SGWrite(_SetLogicalOrigin(Axis)) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function

    ' Return to Logical set origin  
    Public Function ReturnLogicalOrigin(ByVal Axis As Integer) As Boolean
        If Wait_Ready(Axis) = False Then
            Return False
        End If

        If SGWrite(_ReturnLogicalOrigin(Axis)) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Stop stage
    Public Function StopStage(ByVal Axis As Integer) As Boolean
        If SGWrite(_StopStage(Axis)) = False Then
            Return False
        End If

        Return True
    End Function

    ' Stop Emergency command
    Public Function StopStageEmergency() As Boolean
        If SGWrite(_StopStageEmergency()) = False Then
            Return False
        End If

        Return True
    End Function
    ' Remove emergency 
    Public Function RemoveEmergency() As Boolean
        If SGWrite(_RemoveEmergency()) = False Then
            Return False
        End If

        Return True
    End Function

    'Open com port
    Public Function SGOpen() As Boolean
        Try

            If SK_SerialPort.IsOpen = False Then
                SK_SerialPort.Open()
            End If

            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "デバイスOPEN時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the device open."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function

    ' Close Com port
    Public Function SGClose() As Boolean
        Try

            If SK_SerialPort.IsOpen = True Then
                SK_SerialPort.Close()
            End If

            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "デバイスCLOSE時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the device close."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function

    ' Write Command to com port 
    Public Function SGWrite(ByVal Wcmd As String) As Boolean
        Try

            If SK_SerialPort.CtsHolding <> False Then

                If Language = True Then
                    ErrMsg = "送信時にエラーが発生しました。"
                    ErrTitle = "通信エラー"
                Else
                    ErrMsg = "Error occurred at the time of the transmission."
                    ErrTitle = "Communication Error"
                End If

                MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return False
            End If

            If Wcmd = String.Empty Then
                Return False
            End If

            SK_SerialPort.WriteLine(Wcmd)
            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "送信時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the transmission."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function

    ' Read command received in com port
    Public Function SGRead(ByRef Rcmd As String) As Boolean
        Try
            Rcmd = SK_SerialPort.ReadLine()
            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "受信時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the reception."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function
    ' Get Axis direction 
    Private Function GetAxisDir(ByVal Value As Integer) As String
        If Value < 0 Then
            Return "-"
        Else
            Return "+"
        End If
    End Function
    ' Get Axis direction 
    Private Function GetAxisDir(ByVal Value As Double) As String
        If Value < 0 Then
            Return "-"
        Else
            Return "+"
        End If
    End Function

    ' Get Axis direction 
    Private Function GetAxisDir2(ByVal Value As Boolean) As String
        If Value = False Then
            Return "-"
        Else
            Return "+"
        End If
    End Function

    Private Function inGet_Stage_Status() As String
        Dim wkstr As String
        wkstr = String.Empty
        SGWrite(_Status2())
        SGRead(wkstr)
        Return wkstr
    End Function

    ' Get length of string in Q command
    Private Function Get_Qlen(ByVal AxesNum As Integer) As Integer
        Select Case AxesNum
            Case 2
                Return 27
            Case 1
                Return 16
            Case Else
                Return 0
        End Select
    End Function
    ' Home stage command
    Private Function _ReturnOrigin(ByVal Axis As Integer) As String
        If Axis = 0 Then
            Return "H:W"
        ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
            Return "H:" & Axis.ToString()
        Else
            Return ""
        End If
    End Function
    ' Move relative 
    Private Function _MoveRelativity(ByVal Axis As Integer, ByVal Value1 As Integer) As String
        Dim WkStr As String

        If Axis = 1 Then
            WkStr = "M:1" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
            Return WkStr
        ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
            WkStr = "M:" & Axis.ToString() & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function

    Private Function _MoveRelativityW(ByVal Axis As Integer, ByVal Value1 As Integer, ByVal Value2 As Integer) As String
        Dim WkStr As String

        If Axis = 0 Then
            WkStr = "M:W" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString() & GetAxisDir(Value2) & "P" + Math.Abs(Value2).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function
    ' Move Relative  by unit (UNIT = 1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree) 
    Private Function _MoveRelativityByUnit(ByVal Unit As Integer, ByVal Axis As Integer, ByVal Value1 As Double) As String
        Dim WkStr As String

        If Unit = 1 Then

            If Value1 < Resol Then
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return ""
            Else

                If Axis = 1 Then
                    WkStr = "M:1" & GetAxisDir(Value1) & "N" & Math.Abs(Value1).ToString()
                    Return WkStr
                ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                    WkStr = "M:" & Axis.ToString() & GetAxisDir(Value1) & "N" & Math.Abs(Value1).ToString()
                    Return WkStr
                Else
                    Return ""
                End If
            End If
        ElseIf Unit = 2 Then

            If Value1 < Resol / 1000 Then
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return ""
            Else

                If Axis = 1 Then
                    WkStr = "M:1" & GetAxisDir(Value1) & "U" & Math.Abs(Value1).ToString()
                    Return WkStr
                ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                    WkStr = "M:" & Axis.ToString() & GetAxisDir(Value1) & "U" & Math.Abs(Value1).ToString()
                    Return WkStr
                Else
                    Return ""
                End If
            End If
        ElseIf Unit = 3 Then

            If Value1 < Resol / 100000 Then
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return ""
            Else

                If Axis = 1 Then
                    WkStr = "M:1" & GetAxisDir(Value1) & "M" & Math.Abs(Value1).ToString()
                    Return WkStr
                ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                    WkStr = "M:" & Axis.ToString() & GetAxisDir(Value1) & "M" & Math.Abs(Value1).ToString()
                    Return WkStr
                Else
                    Return ""
                End If
            End If
        ElseIf Unit = 4 Then

            If Axis = 1 Then
                WkStr = "M:1" & GetAxisDir(Value1) & "D" & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                WkStr = "M:" & Axis.ToString() & GetAxisDir(Value1) & "D" & Math.Abs(Value1).ToString()
                Return WkStr
            Else
                Return ""
            End If
        Else
            Return ""
        End If
    End Function
    ' Move Absolutely
    Private Function _MoveAbsolutely(ByVal Axis As Integer, ByVal Value1 As Integer) As String
        Dim WkStr As String

        If Axis = 1 Then
            WkStr = "A:1" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
            Return WkStr
        ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
            WkStr = "A:" & Axis.ToString() & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function

    Private Function _MoveAbsolutelyW(ByVal Axis As Integer, ByVal Value1 As Integer, ByVal Value2 As Integer) As String
        Dim WkStr As String

        If Axis = 0 Then
            WkStr = "A:W" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString() & GetAxisDir(Value2) & "P" + Math.Abs(Value2).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function
    ' Move absolue by unit (UNIT = 1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree) 
    Private Function _MoveAbsolutelyByUnit(ByVal Unit As Integer, ByVal Axis As Integer, ByVal Value1 As Integer) As String
        Dim WkStr As String

        If Unit = 1 Then

            If Value1 < Resol Then
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return ""
            Else

                If Axis = 1 Then
                    WkStr = "A:1" & GetAxisDir(Value1) & "N" & Math.Abs(Value1).ToString()
                    Return WkStr
                ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                    WkStr = "A:" & Axis.ToString() & GetAxisDir(Value1) & "N" & Math.Abs(Value1).ToString()
                    Return WkStr
                Else
                    Return ""
                End If
            End If
        ElseIf Unit = 2 Then

            If Value1 < Resol / 1000 Then
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return ""
            Else

                If Axis = 1 Then
                    WkStr = "A:1" & GetAxisDir(Value1) & "U" & Math.Abs(Value1).ToString()
                    Return WkStr
                ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                    WkStr = "A:" & Axis.ToString() & GetAxisDir(Value1) & "U" & Math.Abs(Value1).ToString()
                    Return WkStr
                Else
                    Return ""
                End If
            End If
        ElseIf Unit = 3 Then

            If Value1 < Resol / 100000 Then
                MessageBox.Show(" Value should be equal or bigger than resolution of controller", "Resolution Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
                Return ""
            Else

                If Axis = 1 Then
                    WkStr = "A:1" & GetAxisDir(Value1) & "M" & Math.Abs(Value1).ToString()
                    Return WkStr
                ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                    WkStr = "A:" & Axis.ToString() & GetAxisDir(Value1) & "M" & Math.Abs(Value1).ToString()
                    Return WkStr
                Else
                    Return ""
                End If
            End If
        ElseIf Unit = 4 Then

            If Axis = 1 Then
                WkStr = "A:1" & GetAxisDir(Value1) & "D" & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
                WkStr = "A:" & Axis.ToString() & GetAxisDir(Value1) & "D" & Math.Abs(Value1).ToString()
                Return WkStr
            Else
                Return ""
            End If
        Else
            Return ""
        End If
    End Function
    ' Jog Command 
    Private Function _Jog(ByVal Axis As Integer, ByVal Dir_Renamed As Boolean, ByVal Speed As Integer) As String
        Dim WkStr As String

        If Speed >= 0 And Speed <= 10 Then

            If Axis = 0 Then

                If Speed = 10 Then
                    WkStr = "JY:W" & " " & GetAxisDir2(Dir_Renamed) & "H"
                    Return WkStr
                Else
                    WkStr = "JY:W" & " " & GetAxisDir2(Dir_Renamed) & Speed.ToString()
                    Return WkStr
                End If
            ElseIf Axis = 1 AndAlso Axis <= MaxAxis Then

                If Speed = 10 Then
                    WkStr = "JY:" & Axis.ToString() & " " & GetAxisDir2(Dir_Renamed) & "H"
                    Return WkStr
                Else
                    WkStr = "JY:" & Axis.ToString() & " " & GetAxisDir2(Dir_Renamed) & Speed.ToString()
                    Return WkStr
                End If
            Else
                Return ""
            End If
        Else
            MessageBox.Show(" Speed Value should be between 0 and 10", "Speed Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return ""
        End If
    End Function

    Private Function _Go() As String
        Return "G"
    End Function

    ' Set Logical Origin 
    Private Function _SetLogicalOrigin(ByVal Axis As Integer) As String
        If Axis = 0 Then
            Return "R:W"
        ElseIf Axis = 1 AndAlso Axis <= MaxAxis Then
            Return "R:" & Axis.ToString()
        Else
            Return ""
        End If
    End Function


    Private Function _ReturnLogicalOrigin(ByVal Axis As Integer) As String
        If Axis = 0 Then
            Return "Z:W"
        ElseIf Axis = 1 AndAlso Axis <= MaxAxis Then
            Return "Z:" & Axis.ToString()
        Else
            Return ""
        End If
    End Function


    'Stop emergency 
    Private Function _StopStageEmergency() As String
        Return "L:E"
    End Function
    ' remove the emergency 
    Private Function _RemoveEmergency() As String
        Return "BEC:"
    End Function

    ' Stop stage by axis
    Private Function _StopStage(ByVal Axis As Integer) As String
        If Axis = 0 Then
            Return "L:1"
        ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then
            Return "L:" & Axis.ToString()
        Else
            Return ""
        End If
    End Function
    ' Set speed for both axis
    Private Function _SpeedW(ByVal Axis As Integer, ByVal Fast1 As Double, ByVal Fast2 As Double) As String
        Dim WkStr As String

        If Fast1 < MinSpeed And Fast1 > MaxSpeed And Fast2 < MinSpeed And Fast2 > MaxSpeed Then
            MessageBox.Show(" Speed set should be more than Min Speed and less than Max Speed ", "Speed Error", MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return ""
        End If

        If Axis = 0 Then
            WkStr = "D:W" & "F" & Math.Abs(Fast1).ToString() & "F" + Math.Abs(Fast2).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function
    ' Set speed
    Private Function _Speed(ByVal Axis As Integer, ByVal Fast As Double) As String
        Dim WkStr As String

        If Fast < MinSpeed And Fast > MaxSpeed Then
            Return ""
        End If

        If Axis = 1 AndAlso Axis <= MaxAxis Then
            WkStr = "D:" & Axis.ToString() & "F" & Math.Abs(Fast).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function
    ' Jog command 
    Public Function Jog(ByVal Axis As Integer, ByVal Dir_Renamed As Boolean, ByVal Speed As Integer) As Boolean
        If Wait_Ready(Axis) = False Then
            Return False
        End If

        If SGWrite(_Jog(Axis, Dir_Renamed, Speed)) = False Then
            Return False
        End If

        If Wait_Ready(Axis) = False Then
            Return False
        End If

        Return True
    End Function
    ' Set Speed by unit (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree) ()
    Private Function _SpeedByUnit(ByVal Unit As Integer, ByVal Axis As Integer, ByVal Fast As Double) As String
        Dim WkStr As String

        If Unit = 1 Then

            If Fast < MinSpeed And Fast > MaxSpeed Then
                Return ""
            Else
                WkStr = "D:" & Axis.ToString() & "N" & Math.Abs(Fast).ToString()
                Return WkStr
            End If
        ElseIf Unit = 2 Then

            If Fast < MinSpeed And Fast > MaxSpeed Then
                Return ""
            Else
                WkStr = "D:" & Axis.ToString() & "U" & Math.Abs(Fast).ToString()
                Return WkStr
            End If
        ElseIf Unit = 3 Then

            If Fast < MinSpeed And Fast > MaxSpeed Then
                Return ""
            Else
                WkStr = "D:" & Axis.ToString() & "M" & Math.Abs(Fast).ToString()
                Return WkStr
            End If
        ElseIf Unit = 4 Then

            If Fast < MinSpeed And Fast > MaxSpeed Then
                Return ""
            Else
                WkStr = "D:" & Axis.ToString() & "D" & Math.Abs(Fast).ToString()
                Return WkStr
            End If
        Else
            Return ""
        End If
    End Function
    ' Set Speed for Both Axis 
    Public Function SpeedW(ByVal Axis As Integer, ByVal Fast1 As Double, ByVal Fast2 As Double) As Boolean
        If SGWrite(_SpeedW(Axis, Fast1, Fast2)) = False Then
            Return False
        End If

        Return True
    End Function
    ' Set Speed 
    Public Function Speed(ByVal Axis As Integer, ByVal Fast As Double) As Boolean
        If SGWrite(_Speed(Axis, Fast)) = False Then
            Return False
        End If

        Return True
    End Function
    ' Set Speed by unit (1: Nanometer, 2: Micrometer, 3: Milimeter, 4: Degree) ()
    Public Function SpeedByUnit(ByVal Unit As Integer, ByVal Axis As Integer, ByVal Fast As Double) As Boolean
        If SGWrite(_SpeedByUnit(Unit, Axis, Fast)) = False Then
            Return False
        End If

        Return True
    End Function
    ' Get Q status 
    Private Function _StatusQ() As String
        Return "Q:"
    End Function
    ' Get Q status 

    Public Function StatusQ() As Boolean

        If Wait_Ready(0) = False Then
            Return False
        End If
        If SGWrite(_StatusQ()) = False Then
            Return False
        End If
        If Wait_Ready(0) = False Then
            Return False
        End If
        Return True

    End Function

    ' Get status !:
    Private Function _Status2() As String
        Return "!:"
    End Function

    ' Get status !:
    Public Function Status2() As Boolean

        If Wait_Ready(0) = False Then
            Return False
        End If
        If SGWrite(_Status2()) = False Then
            Return False
        End If
        If Wait_Ready(0) = False Then
            Return False
        End If
        Return True

    End Function

    ' Get Status STS:
    Private Function _Status3(ByVal Axis As Integer) As String
        If Axis = 0 Then
            Return "STS:"
        ElseIf Axis = 1 AndAlso Axis <= MaxAxis Then
            Return "STS:" & Axis
        Else
            Return "Q:"
        End If
    End Function

    ' Get Cordiante Value 
    Private Function Get_Coordinate_Value(ByVal Qstatus As String, ByVal Axis As Integer) As String
        Dim stp As Integer
        Dim BeforeStr As String
        Dim AfterStr As String

        Select Case Axis
            Case 2
                stp = 11
                Exit Select
            Case 0
                stp = 19
                Exit Select
            Case Else
                stp = 1
                Exit Select
        End Select

        If stp = 19 Then
            BeforeStr = Qstatus.Substring(0, stp)
            AfterStr = BeforeStr.Replace(" ", "")
            Return AfterStr
        Else
            BeforeStr = Qstatus.Substring(stp - 1, 10)
            AfterStr = BeforeStr.Replace(" ", "")
            AfterStr = AfterStr.Substring(0, AfterStr.IndexOf(","))
            Return AfterStr
        End If
    End Function
    ' Get Stage Status
    Private Function Get_Stage_Status() As String
        Return inGet_Stage_Status()
    End Function
    ' Get Mode controller : Remote, Teach, Local
    Private Function Get_Mode() As String
        Return "MODE?"
    End Function
    ' Set Remote Mode 
    Private Function Remote_Mode() As String
        Return "MODE:REMOTE"
    End Function
    ' Get controller Type 
    Private Function Get_model() As String
        Return "MODEL:"
    End Function

    ' Set Controller 
    Public Function Set_Model_controller() As Boolean
        Dim Wkstr As String
        Wkstr = String.Empty
        SGWrite(Get_model())

        SGRead(Wkstr)

        If Wkstr = "FC-411" Then
            ControllerType = Controller_Type.FC_411
            Return True
        ElseIf Wkstr = "FC-611" Then
            ControllerType = Controller_Type.FC_611
            Return True
        ElseIf Wkstr = "FC-511" Then
            ControllerType = Controller_Type.FC_511
            Return True
        ElseIf Wkstr = "FC-611" Then
            ControllerType = Controller_Type.FC_611
            Return True
        ElseIf Wkstr = "FC-911" Then
            ControllerType = Controller_Type.FC_911
            Return True
        Else
            Return False
        End If
    End Function



End Class
