﻿//----------------------------------------------------------------------------------------------
//	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
//	This class can be used by anyone provided that the copyright notice remains intact.
//	Program developed by ABED Toufik, checked and confirmed by K.Nobatake.
//
//	This class helper is used to operate SIGMAKOKI Controllers (GSC01, GIP-101B, SHOT302/304GS, Hit, HSC-103, PGC-04). 
//	Use the class to build your own program.
//	We can provide support for the other controllers.
//	Contact us : Phone (+ 81-3-5638-8228) or sales@sigma-koki.com.
//----------------------------------------------------------------------------------------------


using System;
using System.IO.Ports;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Threading;
public partial class SKSampleClass
{
    public enum Controller_Type
    {
        SHOT_304GS,
        Hit_MV,
        PGC_04,
        SHOT_302GS,
        GIP_101B,
        HSC_103,
        GSC01
    }

    public SerialPort SK_SerialPort;
    private Controller_Type Controller;
    private int MaxAxis;
    private string ErrMsg;
    private string ErrTitle;
    private Label PosText;
    public bool Language;
    public bool Hit_mode;

    public SKSampleClass()
    {
        SK_SerialPort = new SerialPort();
        SK_SerialPort.PortName = "COM1";     // Default value
        SK_SerialPort.BaudRate = 9600;       // Default value
        SK_SerialPort.NewLine = "\r\n"; ;      // Default value
        SK_SerialPort.ReadTimeout = 5000;    // Default value
        SK_SerialPort.RtsEnable = true;
        ControllerType = Controller_Type.SHOT_304GS; // Default value
    }


    // Controller Type
    public Controller_Type ControllerType
    {
        get
        {
            return Controller;
        }

        set
        {
            Controller = value;
            if (Controller == Controller_Type.SHOT_302GS)
            {
                MaxAxis = 2;
            }
            else if (Controller == Controller_Type.GIP_101B || Controller == Controller_Type.GSC01)
            {
                MaxAxis = 1;
            }
            else if (Controller == Controller_Type.HSC_103)
            {
                Hit_mode = true;
                MaxAxis = 3;
            }
            else if (Controller == Controller_Type.PGC_04)
            {
                Hit_mode = true;
                MaxAxis = 4;
            }
            else if (Controller == Controller_Type.Hit_MV)
            {
                Hit_mode = true;
                MaxAxis = 4;
            }
            else
            {
                MaxAxis = 4;
            }
        }
    }

    // PortName
    public string PortName
    {
        get
        {
            return SK_SerialPort.PortName;
        }

        set
        {
            SK_SerialPort.PortName = value;
        }
    }

    // Baudrate
    public int Baudrate
    {
        get
        {
            return SK_SerialPort.BaudRate;
        }

        set
        {
            SK_SerialPort.BaudRate = value;
        }
    }

    // Delimiter
    public string Delimiter
    {
        get
        {
            return SK_SerialPort.NewLine;
        }

        set
        {
            SK_SerialPort.NewLine = value;
        }
    }

    // Timeout
    public int Timeout
    {
        get
        {
            return SK_SerialPort.ReadTimeout;
        }

        set
        {
            SK_SerialPort.ReadTimeout = value;
        }
    }

    // PortNames
    public string[] GetPortNames()
    {
        string[] sss;
        sss = SerialPort.GetPortNames();
        return sss;
    }


    // test the conection succes 
    public bool ConnectTest()
    {
        string sss;
        string st;
        string WkStr;
        try
        {

            // デバイスOPEN  device open
            if (SGOpen() == false)
            {
                return false;
            }

            SK_SerialPort.DiscardInBuffer();  // ﾊﾞｯﾌｧｸﾘｱ-->GIP101の場合,最初にｺﾞﾐが入る  In case of buffer clear-> GIP101, the level is entered first

            // Qｺﾏﾝﾄﾞ送信 Send Q command
            if (SGWrite(_StatusQ()) == false)
            {
                return false;
            }

            // 応答受信 response received
            sss = string.Empty;
            if (SGRead(ref sss) == false)
            {
                return false;
            }

            // 受信文字確認  Check received characters
            WkStr = Get_Stage_Status(sss, 3, MaxAxis);
            st = WkStr.ToUpper();
            if (Equals(st, "R") || Equals(st, "B"))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        catch (Exception)
        {
            return false;
        }
    }

    public bool PortClose()
    {
        bool rtn;
        rtn = SGClose();
        return rtn;
    }

    // Wait to Ready
    public bool Wait_Ready()
    {
        string sss;
        string st;
        try
        {
            do
            {
                System.Windows.Forms.Application.DoEvents();
                if (SGWrite(_StatusQ()) == false)
                {
                    return false;
                }

                // 応答受信  response received
                sss = null;
                if (SGRead(ref sss) == false)
                {
                    return false;
                }

                if (!Equals(sss, ""))
                {

                    if (PosText is object)
                    {

                        CurPosLabel.Text = Get_Coordinate_Value(sss, 1);
                    }

                    if (Hit_mode == true)
                    {

                        st = get_status_hit(1);

                        if (Equals(st, "0"))
                        {
                            if (!Equals(st, "1"))
                            {
                                //Thread.Sleep(500);
                                return true;
                            }
                        }
                    }
                    if (Hit_mode == false)
                    {
                        st = Get_Stage_Status(sss, 3, MaxAxis).ToUpper();
                        if (Equals(st, "R"))
                        {
                            return true;
                        }
                    }

                }
            }
            while (true);
        }

        catch (Exception)
        {
            return false;
        }


    }

    public Label CurPosLabel
    {
        get
        {
            return PosText;
        }

        set
        {
            PosText = value;
        }
    }

    // 現在位置   Current position
    public int GetPosition()
    {
        string sss;
        string pos;

        // Ready確認 Confirmation  
        if (Wait_Ready() == false)
        {
            return 0;
        }

        // Qコマンド送信  Q command transmission
        if (SGWrite(_StatusQ()) == false)
        {
            return 0;
        }

        sss = null;
        if (SGRead(ref sss) == false)  // 応答受信   Response received
        {
            return 0;
        }

        pos = Get_Coordinate_Value(sss, MaxAxis); // 座標値を得る  Get coordinate values
        return int.Parse(pos);
    }


    // 機械原点復帰  Mechanical origin return
    public bool ReturnOrigin(int Axis)
    {
        string sss;

        // Ready確認  Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        // Hｺﾏﾝﾄﾞ送信    Send H command
        if (SGWrite(_ReturnOrigin(Axis)) == false)
        {
            return false;
        }

        // 応答受信    response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？ Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }


        // Ready確認  Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    // Set Hit Mode
    public bool HitMode()
    {
        string sss;

        // Hｺﾏﾝﾄﾞ送信    Send Z: command
        if (SGWrite(_HitMode()) == false)
        {
            return false;
        }

        // 応答受信    response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？ Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }


        // Ready確認  Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    // Set Shot mode

    public bool ShotMode()
    {
        string sss;



        // Hｺﾏﾝﾄﾞ送信    Send Z: command
        if (SGWrite(_ShotMode()) == false)
        {
            return false;
        }

        // 応答受信    response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？ Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }


        // Ready確認  Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    public bool MoveA(int Axis, int Vdata)
    {
        string sss;

        // Mコマンド送信  Send M command
        if (SGWrite(_MoveAbsolutely(Axis, Vdata)) == false)
        {
            return false;
        }

        // 応答受信   Response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？  Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        if (Hit_mode == false)
        {
            // Gコマンド送信   G command transmission
            if (SGWrite(_Go()) == false)
            {
                return false;
            }

            // 応答受信 Response received
            sss = null;
            if (SGRead(ref sss) == false)
            {
                return false;
            }

            // 応答がOKか？  Is the response OK?
            if (sss.IndexOf("OK") < 0)
            {
                return false;
            }
        }

        // Ready確認   Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }


    // 移動（相対）  Movement (relative)
    public bool MoveR(int Axis, int Vdata)
    {
        string sss;

        // Mコマンド送信  Send M command
        if (SGWrite(_MoveRelativity(Axis, Vdata)) == false)
        {
            return false;
        }

        // 応答受信   Response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？  Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        if (Hit_mode == false)
        {
            // Gコマンド送信   G command transmission
            if (SGWrite(_Go()) == false)
            {
                return false;
            }

            // 応答受信 Response received
            sss = null;
            if (SGRead(ref sss) == false)
            {
                return false;
            }

            // 応答がOKか？  Is the response OK?
            if (sss.IndexOf("OK") < 0)
            {
                return false;
            }
        }
        // Ready確認   Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    // 論理原点設定  Logical origin setting
    public bool ResetPosition(int Axis)
    {
        string sss;

        // Ready確認  Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        // Rコマンド送信    R command transmission
        if (SGWrite(_ReturnLogicalOrigin(Axis)) == false)
        {
            return false;
        }

        // 応答受信   Response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？  Check if response OK
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        // Ready確認   Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    // 論理原点復帰   Logical origin return
    public bool ReturnLogicalOrigin(int Axis, int cPos = 0)
    {
        string sss;

        // Ready確認   Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        // Mコマンド送信   Send M command
        if (SGWrite(_MoveRelativity(Axis, cPos * -1)) == false)
        {
            return false;
        }

        // 応答受信   Response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？   Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        // Gコマンド送信  G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }

        // 応答受信 response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？ Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        // Ready確認   Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    // 減速停止  Deceleration stop
    public bool StopStage(int Axis)
    {
        string sss;

        // Lｺﾏﾝﾄﾞ送信 Send L command
        if (SGWrite(_StopStage(Axis)) == false)
        {
            return false;
        }

        // 応答受信  response received 
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？ Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        return true;
    }

    // 非常停止   emergency stop
    public bool StopStageEmergency()
    {
        string sss;

        // Lｺﾏﾝﾄﾞ送信  Send L command
        if (SGWrite(_StopStageEmergency()) == false)
        {
            return false;
        }

        // 応答受信  response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？ Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        return true;
    }

    // ﾎﾟｼﾞｼｮﾝ位置書込み(GIP101)  Write Position Position (GIP101)
    public bool SetPosition(int PNo, int Value)
    {
        string sss;

        // Pコマンド送信  P command transmission
        if (SGWrite(_SetPosition(PNo, Value)) == false)
        {
            return false;
        }

        // 応答受信  response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？   Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        return true;
    }

    // ﾎﾟｼﾞｼｮﾝ位置移動(GIP101)  Position position movement (GIP101)
    public bool MovePosition(int PNo)
    {
        string sss;



        // Bコマンド送信  B command transmission
        if (SGWrite(_MovePosition(PNo)) == false)
        {
            return false;
        }

        // 応答受信  response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？  Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        // Gコマンド送信  G command transmission
        if (SGWrite(_Go()) == false)
        {
            return false;
        }

        // 応答受信 response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return false;
        }

        // 応答がOKか？  Is the response OK?
        if (sss.IndexOf("OK") < 0)
        {
            return false;
        }

        // Ready確認  Ready confirmation
        if (Wait_Ready() == false)
        {
            return false;
        }

        return true;
    }

    // ﾎﾟｼﾞｼｮﾝ位置取得(GIP101)  Position position acquisition (GIP101)
    public string StatusP(int PNo)
    {
        string sss;

        // ?コマンド送信  '? Command transmission
        if (SGWrite(_Status3(PNo.ToString())) == false)
        {
            return null;
        }

        // 応答受信  response received
        sss = null;
        if (SGRead(ref sss) == false)
        {
            return null;
        }

        return sss;
    }

    public bool SGOpen()
    {
        try
        {
            if (SK_SerialPort.IsOpen == false)
            {
                SK_SerialPort.Open();
            }

            return true;
        }
        catch (Exception)
        {
            if (Language == true)
            {
                ErrMsg = "デバイスOPEN時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the device open.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }


    // Close
    public bool SGClose()
    {
        try
        {
            if (SK_SerialPort.IsOpen == true)
            {
                SK_SerialPort.Close();
            }

            return true;
        }
        catch (Exception)
        {
            if (Language == true)
            {
                ErrMsg = "デバイスCLOSE時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the device close.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }


    // Write
    public bool SGWrite(string Wcmd)
    {
        try
        {

            // If CtlClass.CtsHoldingChkFlg = True Then

            if (SK_SerialPort.CtsHolding != true)
            {
                if (Language == true)
                {
                    ErrMsg = "送信時にエラーが発生しました。";
                    ErrTitle = "通信エラー";
                }
                else
                {
                    ErrMsg = "Error occurred at the time of the transmission.";
                    ErrTitle = "Communication Error";
                }

                MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            // End If

            // SK_SerialPort.Write(Wcmd)
            SK_SerialPort.WriteLine(Wcmd);
            return true;
        }
        catch (Exception)
        {
            if (Language == true)
            {
                ErrMsg = "送信時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the transmission.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }


    // Read
    public bool SGRead(ref string Rcmd)
    {
        try
        {
            Rcmd = SK_SerialPort.ReadLine();
            return true;
        }
        catch (Exception)
        {
            if (Language == true)
            {
                ErrMsg = "受信時にエラーが発生しました。";
                ErrTitle = "通信エラー";
            }
            else
            {
                ErrMsg = "Error occurred at the time of the reception.";
                ErrTitle = "Communication Error";
            }

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            return false;
        }
    }




    // 移動方向（内部関数）Moving direction (internal function)
    private string GetAxisDir(int Value)
    {
        if (Value < 0)
        {
            return "-";
        }
        else
        {
            return "+";
        }
    }


    // 移動方向2（内部関数） Moving direction 2 (internal function)
    private string GetAxisDir2(bool Value)
    {
        if (Value == false)
        {
            return "-";
        }
        else
        {
            return "+";
        }
    }


    // Qｺﾏﾝﾄﾞから座標値を取出す   Extract coordinate values from Q command
    private string inGet_Coordinate_Value(string Qstatus, int AxesNum, int Axis)
    {
        int stp;
        int AxLen;
        string BeforeStr;
        string AfterStr;
        AxLen = Get_Qlen(AxesNum); // 文字列長    String length


        if (Hit_mode == true)
        {
            switch (Axis)
            {
                case 4:
                    {
                        stp = 3;
                        break;
                    }

                case 3:
                    {
                        stp = 2;
                        break;
                    }

                case 2:
                    {
                        stp = 1;
                        break;
                    }

                default:
                    {
                        stp = 0;
                        break;
                    }
            }
            //BeforeStr = Qstatus;
            //BeforeStr = Regex.Replace(Qstatus, @"\D", "");
            int kk = Qstatus.Length;
            BeforeStr = Qstatus.Substring(stp, kk - stp); // -1:1文字目のindexは0のため  -1: Since the index of the first character is 0
            BeforeStr = BeforeStr.Substring(0, BeforeStr.IndexOf(","));
            AfterStr = BeforeStr.Replace(" ", "");
            return AfterStr;
        }
        else
        {

            // Qｺﾏﾝﾄﾞの長さ   Q command length
            if (Qstatus.Length < AxLen)
            {
                return "0";
            }

            // 取出す先頭  Retrieving head
            switch (Axis)
            {
                case 4:
                    {
                        stp = 34;
                        break;
                    }

                case 3:
                    {
                        stp = 23;
                        break;
                    }

                case 2:
                    {
                        stp = 12;
                        break;
                    }

                default:
                    {
                        stp = 1;
                        break;
                    }
            }

            BeforeStr = Qstatus.Substring(stp - 1, 10); // -1:1文字目のindexは0のため  -1: Since the index of the first character is 0
            AfterStr = BeforeStr.Replace(" ", "");
            return AfterStr;
        }
        // return Qstatus.Substring(stp - 1, 10); //-1:1文字目のindexは0のため
    }


    // Qｺﾏﾝﾄﾞからｽﾃｰﾀｽを取出す  Extract status from Q-command
    private string inGet_Stage_Status(string Qstatus, int AxesNum, int ACKNumber)
    {
        int stp;
        int AxLen;
        AxLen = Get_Qlen(AxesNum); // 文字列長  String length




        // Qｺﾏﾝﾄﾞの長さ  Q command length
        if (Qstatus.Length < AxLen)
        {
            return "0";
        }

        // 取出す先頭  Retrieving head
        switch (AxesNum)
        {
            case 4:   // 4軸    4axis
                {
                    switch (ACKNumber)
                    {
                        case 1:
                            {
                                stp = 45;
                                break;
                            }

                        case 2:
                            {
                                stp = 47;
                                break;
                            }

                        default:
                            {
                                stp = 49;
                                break;
                            }
                    }

                    break;
                }

            case 3:  // 3軸 3axis
                {
                    switch (ACKNumber)
                    {
                        case 1:
                            {
                                stp = 34;
                                break;
                            }

                        case 2:
                            {
                                stp = 36;
                                break;
                            }

                        default:
                            {
                                stp = 38;
                                break;
                            }
                    }

                    break;
                }

            case 2:  // 2軸 2axis
                {
                    switch (ACKNumber)
                    {
                        case 1:
                            {
                                stp = 23;
                                break;
                            }

                        case 2:
                            {
                                stp = 25;
                                break;
                            }

                        default:
                            {
                                stp = 27;
                                break;
                            }
                    } // 1軸  1axis

                    break;
                }

            default:
                {
                    switch (ACKNumber)
                    {
                        case 1:
                            {
                                stp = 12;
                                break;
                            }

                        case 2:
                            {
                                stp = 14;
                                break;
                            }

                        default:
                            {
                                stp = 16;
                                break;
                            }
                    }

                    break;
                }
        }

        return Qstatus.Substring(stp - 1, 1); // -1:1文字目のindexは0のため   -1: Since the index of the first character is 0
    }


    // Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command
    private int Get_Qlen(int AxesNum)
    {
        switch (AxesNum) // 軸数  Number of axes
        {
            case 4:
                {
                    return 49;
                }

            case 3:
                {
                    return 38;
                }

            case 2:
                {
                    return 27;
                }

            case 1:
                {
                    return 16;
                }

            default:
                {
                    return 0;
                }
        }
    }


    // Return origin
    private string _ReturnOrigin(int Axis)
    {
        if (Hit_mode == true)
        {
            if (Axis == 0 || Axis == 1)  // All axis
            {
                return "H:1";
            }
            else if (Axis == 2)  // Each axis
            {
                return "H:,1";
            }
            else if (Axis == 3)  // Each axis
            {
                return "H:,,1";
            }
            else if (Axis == 4)  // Each axis
            {
                return "H:,,,1";
            }
            else
            {
                return "";
            }

        }
        else
        {
            if (Axis == 0)  // All axis
            {
                return "H:1";
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                return "H:" + Axis.ToString();
            }
            else
            {
                return "";
            }

        }
    }


    // Move relativity
    private string _MoveRelativity(int Axis, int Value1)
    {
        string WkStr;
        if (Hit_mode == true)
        {
            if (Axis == 0 || Axis == 1)  // All axis
            {
                WkStr = "M:" + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            if (Axis == 2)  // All axis
            {
                WkStr = "M:," + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            if (Axis == 3)  // All axis
            {
                WkStr = "M:,," + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            if (Axis == 4)  // All axis
            {
                WkStr = "M:,,," + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
        else
        {
            if (Axis == 0)  // All axis
            {
                WkStr = "M:1" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
    }


    // Move absolutely
    private string _MoveAbsolutely(int Axis, int Value1)
    {

        string WkStr;
        if (Hit_mode == true)
        {
            if (Axis == 0 || Axis == 1)
            {
                WkStr = "A:" + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            if (Axis == 2)
            {
                WkStr = "A:," + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            if (Axis == 3)
            {
                WkStr = "A:,," + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            if (Axis == 4)
            {
                WkStr = "A:,,," + GetAxisDir(Value1) + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
        else
        {
            if (Axis == 0)  // All axis
            {
                WkStr = "A:1" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
    }


    // Move with JOG
    private string _Jog(int Axis, bool Dir_Renamed)
    {
        string WkStr;
        if (Hit_mode == true)
        {
            if (Axis == 0 || Axis == 1)  // All axis
            {
                WkStr = "J:" + GetAxisDir2(Dir_Renamed);
                return WkStr;
            }
            else if (Axis == 2)  // All axis
            {
                WkStr = "J:," + GetAxisDir2(Dir_Renamed);
                return WkStr;
            }
            else if (Axis == 3)  // All axis
            {
                WkStr = "J:,," + GetAxisDir2(Dir_Renamed);
                return WkStr;
            }
            else if (Axis == 4)  // All axis
            {
                WkStr = "J:,,," + GetAxisDir2(Dir_Renamed);
                return WkStr;
            }
            else
            {
                return "";
            }

        }
        else
        {
            if (Axis == 0)  // All axis
            {
                WkStr = "J:1" + GetAxisDir2(Dir_Renamed);
                return WkStr;
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                WkStr = "J:" + Axis.ToString() + GetAxisDir2(Dir_Renamed);
                return WkStr;
            }
            else
            {
                return "";
            }
        }
    }


    // Drive
    private string _Go()
    {
        return "G:";
    }


    // Return logical origin
    private string _ReturnLogicalOrigin(int Axis)
    {
        if (Hit_mode == true)
        {
            if (Axis == 0 || Axis == 1)  // All axis
            {
                return "R:1";
            }
            else if (Axis == 2)  // Each axis
            {
                return "R:,1";
            }
            else if (Axis == 3)  // Each axis
            {
                return "R:,,1";
            }
            else if (Axis == 4)  // Each axis
            {
                return "R:,,,1";
            }
            else
            {
                return "";
            }


        }
        else
        {
            if (Axis == 0)  // All axis
            {
                return "R:1";
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                return "R:" + Axis.ToString();
            }
            else
            {
                return "";
            }
        }
    }


    // Move Position
    private string _MovePosition(int BNo)
    {
        if (BNo == 0)  // All axis
        {
            return "B:1";
        }
        else if (BNo >= 1 && BNo <= 5)  // 1-5
        {
            return "B:" + BNo.ToString();
        }
        else
        {
            return "";
        }
    }


    // Set Position
    private string _SetPosition(int BNo, int Value1)
    {
        string WkStr;
        if (BNo == 0)  // All axis
        {
            WkStr = "P:B1" + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
            return WkStr;
        }
        else if (BNo >= 1 && BNo <= 5)  // 1-5
        {
            WkStr = "P:B" + BNo.ToString() + GetAxisDir(Value1) + "P" + Math.Abs(Value1).ToString();
            return WkStr;
        }
        else
        {
            return "";
        }
    }


    // Emergency stop
    private string _StopStageEmergency()
    {
        return "L:E";
    }


    // Stop
    private string _StopStage(int Axis)
    {
        if (Hit_mode == true)
        {
            if (Axis == 0 || Axis == 1)  // All axis
            {
                return "L:1";
            }
            else if (Axis == 2)  // Each axis
            {
                return "L:,1";
            }
            else if (Axis == 3)  // Each axis
            {
                return "L:,,1";
            }
            else if (Axis == 4)  // Each axis
            {
                return "L:,,,1";
            }
            else
            {
                return "";
            }

        }
        else
        {
            if (Axis == 0)  // All axis
            {
                return "L:1";
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                return "L:" + Axis.ToString();
            }
            else
            {
                return "";
            }
        }
    }


    // Speed
    private string _Speed(int Axis, int Slow, int Fast, int Rate_Renamed)
    {
        string WkStr;
        if (Hit_mode == true)
        {
            if (Slow < 1 || Slow > 999999999 || Fast < 1 || Fast > 999999999 || Rate_Renamed < 0 || Rate_Renamed > 1000)
            {
                return "";
            }
            if (Axis == 0 || Axis == 1)  // All axis
            {
                WkStr = "D:1" + Math.Abs(Slow).ToString() + Math.Abs(Fast).ToString() + Math.Abs(Rate_Renamed).ToString();
                return WkStr;
            }
            if (Axis == 2 || Axis <= MaxAxis)  // All axis
            {
                WkStr = "D:" + Axis.ToString() + Math.Abs(Slow).ToString() + Math.Abs(Fast).ToString() + Math.Abs(Rate_Renamed).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }

        }
        else
        {

            if (Slow < 1 || Slow > 500000 || Fast < 1 || Fast > 500000 || Rate_Renamed < 0 || Rate_Renamed > 1000)
            {
                return "";
            }

            if (Axis == 0)  // All axis
            {
                WkStr = "D:W" + "S" + Math.Abs(Slow).ToString() + "F" + Math.Abs(Fast).ToString() + "R" + Math.Abs(Rate_Renamed).ToString();
                return WkStr;
            }
            else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
            {
                WkStr = "D:" + Axis.ToString() + "S" + Math.Abs(Slow).ToString() + "F" + Math.Abs(Fast).ToString() + "R" + Math.Abs(Rate_Renamed).ToString();
                return WkStr;
            }
            else
            {
                return "";
            }
        }
    }

    public bool Speed(int Axis, int Slow, int Fast, int Rate_Renamed)
    {

        if (SGWrite(_Speed(Axis, Slow, Fast, Rate_Renamed)) == false)
        {
            return false;
        }
        else
        {
            return true;
        }

    }

    // Get the status "Q"
    private string _StatusQ()
    {
        return "Q:";
    }


    // Get the status "!"
    private string _Status2()
    {
        return "!:";
    }

    // Get the status "?"
    public string _Status3(string Para)
    {
        return "?:" + Para;
    }

    // Get the coordinate value from the status "Q"
    private string Get_Coordinate_Value(string Qstatus,int Axis =1)
    {
        return inGet_Coordinate_Value(Qstatus, 1, 1);
    }


    // Get the status from the status "Q"
    private string Get_Stage_Status(string Qstatus, int ACKNumber, int Axis = 1)
    {
        return inGet_Stage_Status(Qstatus, Axis, ACKNumber);
    }

    private string get_status_hit(int Axis)
    {
        string Wkl;
        Wkl = string.Empty;
        SGWrite(_Status2());
        Thread.Sleep(100);
        SGRead( ref Wkl);
        int stp;
        switch (Axis)
        {
            case 4:
                {
                    stp = 3;
                    break;
                }

            case 3:
                {
                    stp = 2;
                    break;
                }

            case 2:
                {
                    stp = 1;
                    break;
                }

            default:
                {
                    stp = 0;
                    break;
                }
        }
        //Wkl = Regex.Replace(Wkl, @"\D", "");
        int kk = Wkl.Length;
        Wkl = Wkl.Substring(stp, kk-stp);
        Wkl = Wkl.Substring(0, Wkl.IndexOf(","));
        return Wkl;
    }


    // Hit Mode 
    private string _HitMode()
    {
        return "Z:1";
    }
    // Shot Mode 
    private string _ShotMode()
    {
        return "Z:0";
    }

}