﻿//----------------------------------------------------------------------------------------------
//	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
//	This class can be used by anyone provided that the copyright notice remains intact.
//	Program developed by ABED Toufik, checked and confirmed by K.Nobatake.
//
//	This class helper is used to operate SIGMAKOKI Controllers (GSC01, GIP-101B, SHOT302/304GS, Hit, HSC-103, PGC-04).
//	Use the class to build your own program.
//	We can provide support for the other controllers.
//	Contact us : Phone (+ 81-3-5638-8228) or sales@sigma-koki.com.
//----------------------------------------------------------------------------------------------

#include <windows.h>
#include "SKSampleClass.h"


using namespace System::Windows::Forms;

SKSampleClass::SKSampleClass()
{
	SK_SerialPort = gcnew SerialPort();
	SK_SerialPort->PortName = L"COM1"; // Default value
	SK_SerialPort->BaudRate = 9600; // Default value
	SK_SerialPort->NewLine = L"\r\n";
	; // Default value
	SK_SerialPort->ReadTimeout = 5000; // Default value
	SK_SerialPort->RtsEnable = true;
	setControllerType(Controller_Type::SHOT_304GS); // Default value
}

SKSampleClass::Controller_Type SKSampleClass::getControllerType() 
{
	return Controller;
}

void SKSampleClass::setControllerType(Controller_Type value)
{
	Controller = value;
	if (Controller == Controller_Type::SHOT_302GS)
	{
		MaxAxis = 2;
	}
	else if (Controller == Controller_Type::GIP_101B || Controller == Controller_Type::GSC01)
	{
		MaxAxis = 1;
	}
	else if (Controller == Controller_Type::HSC_103)
	{
		Hit_mode = true;
		MaxAxis = 3;
	}
	else if (Controller == Controller_Type::PGC_04)
	{
		Hit_mode = true;
		MaxAxis = 4;
	}
	else if (Controller == Controller_Type::Hit_MV)
	{
		Hit_mode = true;
		MaxAxis = 4;
	}
	else
	{
		MaxAxis = 4;
	}
}

String^ SKSampleClass::getPortName()
{
	return SK_SerialPort->PortName;
}

void SKSampleClass::setPortName(String^ &value)
{
	SK_SerialPort->PortName = value;
}

int SKSampleClass::getBaudrate() 
{
	return SK_SerialPort->BaudRate;
}

void SKSampleClass::setBaudrate(int value)
{
	SK_SerialPort->BaudRate = value;
}

String^ SKSampleClass::getDelimiter()
{
	return SK_SerialPort->NewLine;
}

void SKSampleClass::setDelimiter(String^ &value)
{
	SK_SerialPort->NewLine = value;
}

int SKSampleClass::getTimeout() 
{
	return SK_SerialPort->ReadTimeout;
}

void SKSampleClass::setTimeout(int value)
{
	SK_SerialPort->ReadTimeout = value;
}

array<Object^>^ SKSampleClass::GetPortNames()
{
	array<Object^>^ sss;
	sss = SK_SerialPort->GetPortNames();
	return sss;
}

bool SKSampleClass::ConnectTest()
{
	String^ sss;
	String^ st;
	String^ WkStr;
	try
	{

		// デバイスOPEN  device open
		if (SGOpen() == false)
		{
			return false;
		}

		SK_SerialPort->DiscardInBuffer(); // ﾊﾞｯﾌｧｸﾘｱ-->GIP101の場合,最初にｺﾞﾐが入る  In case of buffer clear-> GIP101, the level is entered first

		// Qｺﾏﾝﾄﾞ送信 Send Q command
		if (SGWrite(_StatusQ()) == false)
		{
			return false;
		}

		// 応答受信 response received
		sss = "";
		if (SGRead(sss) == false)
		{
			return false;
		}

		// 受信文字確認  Check received characters
		WkStr = Get_Stage_Status(sss, 3, MaxAxis);
		st = WkStr->ToUpper();
		
		if ((st == L"R") || (st == L"B"))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	catch (UnauthorizedAccessException^)
	{
		return false;
	}
}

bool SKSampleClass::PortClose()
{
	bool rtn;
	rtn = SGClose();
	return rtn;
}

// Wait Ready 
bool SKSampleClass::Wait_Ready()
{
	String^ sss;
	String^ st;
	try
	{
		do
		{
			System::Windows::Forms::Application::DoEvents();
			if (SGWrite(_StatusQ()) == false)
			{
				return false;
			}

			// 応答受信  response received
			sss = String::Empty;
			if (SGRead(sss) == false)
			{
				return false;
			}

			if ((sss != ""))
			{
				
				getCurPosLabel()->Text = Get_Coordinate_Value(sss, 1);
				

				if (Hit_mode == true)
				{
					st = get_status_hit(1);

					if ((st == "0"))
					{
						return true;
					}
				}
				if (Hit_mode == false)
				{
					st = (Get_Stage_Status(sss, 3, MaxAxis))->ToUpper();
				
						if ((st == "R"))
						{
							return true;
						}
				}
			}
		} while (true);
	}

	catch (UnauthorizedAccessException^)
	{
		return false;
	}


}

// Get Label position
Label ^SKSampleClass::getCurPosLabel()
{
	return PosText;
}
// set Label Position
void SKSampleClass::setCurPosLabel(Label ^value)
{
	PosText = value;
}

// get position 
int SKSampleClass::GetPosition()
{
	String^  sss;
	String^  pos;

	// Ready確認 Confirmation  
	if (Wait_Ready() == false)
	{
		return 0;
	}

	// Qコマンド送信  Q command transmission
	if (SGWrite(_StatusQ()) == false)
	{
		return 0;
	}

	sss = L"";
	if (SGRead(sss) == false) // 応答受信   Response received
	{
		return 0;
	}

	pos = Get_Coordinate_Value(sss, MaxAxis); // 座標値を得る  Get coordinate values
	return System::Int32::Parse(pos);
}

// 機械原点復帰  Mechanical origin return
bool SKSampleClass::ReturnOrigin(int Axis)
{

	String^ sss;

	// Ready確認 Confirmation  
	if (Wait_Ready() == false)
	{
		return false;
	}
	// Hｺﾏﾝﾄﾞ送信    Send H command
	if (SGWrite(_ReturnOrigin(Axis)) == false)
	{
		return false;
	}
	// 応答受信 response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	if (sss->IndexOf("OK") <0 )
	{
		return false;
	}

	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;
}

// Hit Mode 
bool SKSampleClass::HitMode()
{

	String^ sss;

	// Zｺﾏﾝﾄﾞ送信    Send Z command
	if (SGWrite(_HitMode()) == false)
	{
		return false;
	}
	// 応答受信 response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	if (sss->IndexOf("OK") <0)
	{
		return false;
	}

	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;
}

//Shot Mode
bool SKSampleClass::ShotMode()
{

	String^ sss;

	// Zｺﾏﾝﾄﾞ送信    Send Z command
	if (SGWrite(_ShotMode()) == false)
	{
		return false;
	}
	// 応答受信 response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	if (sss->IndexOf("OK") <0)
	{
		return false;
	}

	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;
}

//Return Origin 

String^ SKSampleClass::_ReturnOrigin(int Axis)
{
	if (Hit_mode == true)
	{
		if (Axis == 0 || Axis == 1)  // All axis
		{
			return "H:1";
		}
		else if (Axis == 2)  // Each axis
		{
			return "H:,1";
		}
		else if (Axis == 3)  // Each axis
		{
			return "H:,,1";
		}
		else if (Axis == 4)  // Each axis
		{
			return "H:,,,1";
		}
		else
		{
			return "";
		}

	}
	else
	{
		if (Axis == 0)  // All axis
		{
			return "H:1";
		}
		else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
		{
			return "H:" + Axis.ToString();
		}
		else
		{
			return "";
		}

	}

}

// Write 
bool SKSampleClass::SGWrite(String^ Wcmd)
{
	try
	{
		if (SK_SerialPort->CtsHolding != true)
		{
			if (Language == true)
			{
				ErrMsg = "送信時にエラーが発生しました。";
				ErrTitle = "通信エラー";
			}
			else
			{
				ErrMsg = "Error occurred at the time of the transmission.";
				ErrTitle = "Communication Error";
			}
			MessageBox::Show( ErrMsg, ErrTitle);
			return false;
		}

		SK_SerialPort->WriteLine(Wcmd);
		return true;

	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "送信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the transmission.";
			ErrTitle = "Communication Error";
		}
		MessageBox::Show(ErrMsg, ErrTitle);
		return false;

	}
}

//read 
bool SKSampleClass::SGRead( String^ &Rcmd)
{
	try
	{
		Rcmd= SK_SerialPort->ReadLine();
		return true;

	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "受信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the reception.";
			ErrTitle = "Communication Error";
		}

		MessageBox::Show(ErrMsg, ErrTitle);
		return false;
	}


}

//status Q

String^ SKSampleClass::_StatusQ()
{
	return "Q:";
}

//Open

bool SKSampleClass::SGOpen()
{
	try
	{
		if (SK_SerialPort->IsOpen == false) 
		{

			SK_SerialPort->Open();
		}
		return true;

	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "受信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the reception.";
			ErrTitle = "Communication Error";
		}

		MessageBox::Show(ErrMsg, ErrTitle);
		return false;
	}
}
//Close 

bool SKSampleClass::SGClose()
{
	try
	{
		if (SK_SerialPort->IsOpen == true) 
		{

			SK_SerialPort->Close();
		}

		return true;
	}
	catch (const std::exception&)
	{
		if (Language == true)
		{
			ErrMsg = "受信時にエラーが発生しました。";
			ErrTitle = "通信エラー";
		}
		else
		{
			ErrMsg = "Error occurred at the time of the reception.";
			ErrTitle = "Communication Error";
		}

		MessageBox::Show(ErrMsg, ErrTitle);
		return false;

	}
}

// Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command

int SKSampleClass::Get_Qlen(int AxisNum)
{
	switch (AxisNum)
	{
		case 4:
		{
			return 49;
		}
		case 3:
		{
			return 38;
		}
		case 2:
		{
			return 27;
		}
		case 1:
		{
			return 16;
		}
		default:
		{
			return 0;
		}
	}



}

// Qｺﾏﾝﾄﾞからｽﾃｰﾀｽを取出す  Extract status from Q-command
String^ SKSampleClass::inGet_Stage_Status(String^ Qstatus, int AxesNum, int ACKNumber)
{

	int stp;
	int AxLen;

	AxLen = Get_Qlen(AxesNum);
	// 文字列長  String length
	// Qｺﾏﾝﾄﾞの長さ  Q command length
	if (Qstatus->Length < AxLen)
	{
		return "0";
	}
	// 取出す先頭  Retrieving head
	// 取出す先頭  Retrieving head
	switch (AxesNum)
	{
	case 4:   // 4軸    4axis
	{
		switch (ACKNumber)
		{
		case 1:
		{
			stp = 45;
			break;
		}

		case 2:
		{
			stp = 47;
			break;
		}

		default:
		{
			stp = 49;
			break;
		}
		}

		break;
	}

	case 3:  // 3軸 3axis
	{
		switch (ACKNumber)
		{
		case 1:
		{
			stp = 34;
			break;
		}

		case 2:
		{
			stp = 36;
			break;
		}

		default:
		{
			stp = 38;
			break;
		}
		}

		break;
	}

	case 2:  // 2軸 2axis
	{
		switch (ACKNumber)
		{
		case 1:
		{
			stp = 23;
			break;
		}

		case 2:
		{
			stp = 25;
			break;
		}

		default:
		{
			stp = 27;
			break;
		}
		} // 1軸  1axis

		break;
	}

	default:
	{
		switch (ACKNumber)
		{
		case 1:
		{
			stp = 12;
			break;
		}

		case 2:
		{
			stp = 14;
			break;
		}

		default:
		{
			stp = 16;
			break;
		}
		}

		break;
	}
	}
	return Qstatus->Substring(stp - 1, 1); // -1:1文字目のindexは0のため   -1: Since the index of the first character is 0

}

// Get the status from the status "Q"
String ^ SKSampleClass::Get_Stage_Status(String^ Qstatus, int ACKNumber, int Axis)
{

	return inGet_Stage_Status(Qstatus, Axis, ACKNumber);

}

// Qｺﾏﾝﾄﾞから座標値を取出す   Extract coordinate values from Q command
String^ SKSampleClass::inGet_Coordinate_Value(String^ Qstatus, int AxesNum, int Axis)
{
	int stp;
	int AxLen;
	String^ BeforeStr;
	String^ AfterStr;
	AxLen = Get_Qlen(AxesNum); // 文字列長    String length

	if ( Hit_mode == true)
	{
		switch (Axis)
		{
		case 4:
		{
			stp = 3;
			break;
		}

		case 3:
		{
			stp = 2;
			break;
		}

		case 2:
		{
			stp = 1;
			break;
		}

		default:
		{
			stp = 0;
			break;
		}
		}

	int kk = Qstatus->Length;
	BeforeStr = Qstatus->Substring(stp, kk - stp); // -1:1文字目のindexは0のため  -1: Since the index of the first character is 0
	BeforeStr = BeforeStr->Substring(0, BeforeStr->IndexOf(","));
	AfterStr = BeforeStr->Replace(" ", "");
	return AfterStr;



	}
	else
	{
					   
							   // Qｺﾏﾝﾄﾞの長さ   Q command length
	if (Qstatus->Length < AxLen)
	{
		return "0";
	}
	// 取出す先頭  Retrieving head
	switch (Axis)
	{
	case 4:
	{
		stp = 34;
		break;
	}

	case 3:
	{
		stp = 23;
		break;
	}

	case 2:
	{
		stp = 12;
		break;
	}

	default:
	{
		stp = 1;
		break;
	}
	}

	BeforeStr = Qstatus->Substring(stp - 1, 10); // -1:1文字目のindexは0のため  -1: Since the index of the first character is 0
	AfterStr = BeforeStr->Replace(" ", "");
	return AfterStr;
	}


}

// Get the coordinate value from the status "Q"
String^  SKSampleClass::Get_Coordinate_Value(String^  Qstatus, int Axis)
{

	return inGet_Coordinate_Value(Qstatus, 1, 1);

}

String^ SKSampleClass::get_status_hit(int Axis)
{
	String^ Wkl;

	SGWrite(_Status2());
	Thread::Sleep(100);
	SGRead(Wkl);
	int stp;
	switch (Axis)
	{
	case 4:
	{
		stp = 3;
		break;
	}

	case 3:
	{
		stp = 2;
		break;
	}

	case 2:
	{
		stp = 1;
		break;
	}

	default:
	{
		stp = 0;
		break;
	}
	}

	int kk = Wkl->Length;
	Wkl = Wkl->Substring(stp, kk - stp);
	Wkl = Wkl->Substring(0, Wkl->IndexOf(","));
	return Wkl;

}



// Move relative
String^ SKSampleClass::_MoveRelativity(int Axis, int Value1)
{
	String^ WkStr;
	if (Hit_mode == true)
	{
		if (Axis == 0 || Axis == 1)  // All axis
		{
			WkStr = "M:" + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		if (Axis == 2)  // All axis
		{
			WkStr = "M:," + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		if (Axis == 3)  // All axis
		{
			WkStr = "M:,," + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		if (Axis == 4)  // All axis
		{
			WkStr = "M:,,," + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else
		{
			return "";
		}
	}
	else
	{
		if (Axis == 0)  // All axis
		{
			WkStr = "M:1" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
		{
			WkStr = "M:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else
		{
			return "";
		}
	}


}

// move absolute
String^ SKSampleClass::_MoveAbsolutely(int Axis, int Value1)
{
	String^ WkStr;
	if (Hit_mode == true)
	{
		if (Axis == 0 || Axis == 1)
		{
			WkStr = "A:" + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		if (Axis == 2)
		{
			WkStr = "A:," + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		if (Axis == 3)
		{
			WkStr = "A:,," + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		if (Axis == 4)
		{
			WkStr = "A:,,," + GetAxisDir(Value1) + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else
		{
			return "";
		}
	}
	else
	{
		if (Axis == 0)  // All axis
		{
			WkStr = "A:1" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
		{
			WkStr = "A:" + Axis.ToString() + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
			return WkStr;
		}
		else
		{
			return "";
		}
	}


}

//get direction 
String^ SKSampleClass::GetAxisDir(int Value)
{
	if (Value <0)
	{
		return "-";
	}
	else
	{
		return "+";
	}

}

//get Bool direction
String^ SKSampleClass::GetAxisDir2(bool Value)
{
	if (Value == false)
	{
		return "-";
	}
	else
	{
		return "+";
	}
}

// Jog command 
String^ SKSampleClass::_Jog(int Axis, bool Dir_Renamed)
{
	String^ WkStr;
	if (Axis == 0)  // All axis
	{
		WkStr = "J:1" + GetAxisDir2(Dir_Renamed);
		return WkStr;
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
	{
		WkStr = "J:" + Axis.ToString() + GetAxisDir2(Dir_Renamed);
		return WkStr;
	}
	else
	{
		return "";
	}

}

// Go command
String^ SKSampleClass::_Go()
{

	return "G:";
}

// return Logical Origin
String^ SKSampleClass::_ReturnLogicalOrigin(int Axis)
{
	if (Hit_mode == true)
	{
		if (Axis == 0 || Axis == 1)  // All axis
		{
			return "R:1";
		}
		else if (Axis == 2)  // Each axis
		{
			return "R:,1";
		}
		else if (Axis == 3)  // Each axis
		{
			return "R:,,1";
		}
		else if (Axis == 4)  // Each axis
		{
			return "R:,,,1";
		}
		else
		{
			return "";
		}


	}
	else
	{
		if (Axis == 0)  // All axis
		{
			return "R:1";
		}
		else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
		{
			return "R:" + Axis.ToString();
		}
		else
		{
			return "";
		}
	}

}

// Move to position (GIP-101B controller) 
String^ SKSampleClass::_MovePosition(int BNo)
{
	if (BNo == 0)  // All axis
	{
		return "B:1";
	}
	else if (BNo >= 1 && BNo <= 5)  // 1-5
	{
		return "B:" + BNo.ToString();
	}
	else
	{
		return "";
	}
}

// Set position (GIP-101B controller) 
String^ SKSampleClass::_SetPosition(int BNo, int Value1)
{
	String^ WkStr;
	if (BNo == 0)  // All axis
	{
		WkStr = "P:B1" + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
		return WkStr;
	}
	else if (BNo >= 1 && BNo <= 5)  // 1-5
	{
		WkStr = "P:B" + BNo.ToString() + GetAxisDir(Value1) + "P" + Math::Abs(Value1).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}


}

// Emergency Stop
String^ SKSampleClass::_StopStageEmergency()
{
	return "L:E";

}

// Stop stages by axis 
String^ SKSampleClass::_StopStage(int Axis)
{
	if (Axis == 0)  // All axis
	{
		return "L:1";
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
	{
		return "L:" + Axis.ToString();
	}
	else
	{
		return "";
	}

}

// Set Speed
String^ SKSampleClass::_Speed(int Axis, int Slow, int Fast, int Rate_Renamed)
{
	String^ WkStr;
	if (Slow < 1 || Slow > 500000 || Fast < 1 || Fast > 500000 || Rate_Renamed < 0 || Rate_Renamed > 1000)
	{
		return "";
	}

	if (Axis == 0)  // All axis
	{
		WkStr = "D:W" + "S" + Math::Abs(Slow).ToString() + "F" + Math::Abs(Fast).ToString() + "R" + Math::Abs(Rate_Renamed).ToString();
		return WkStr;
	}
	else if (Axis >= 1 && Axis <= MaxAxis)  // Each axis
	{
		WkStr = "D:" + Axis.ToString() + "S" + Math::Abs(Slow).ToString() + "F" + Math::Abs(Fast).ToString() + "R" + Math::Abs(Rate_Renamed).ToString();
		return WkStr;
	}
	else
	{
		return "";
	}


}

// Get the status "!"
String^ SKSampleClass::_Status2()
{
	return "!:";

}

// Get the status "?"

String^ SKSampleClass::_Status3(String^ Para)
{
	return "?:" + Para;

}

// 移動（相対）  Movement (relative)
bool SKSampleClass::MoveR(int Axis, int Vdata)
{
	String^ sss;

	// Mコマンド送信  Send M command
	if (SGWrite(_MoveRelativity(Axis, Vdata)) == false)
	{
		return false;
	}

	// 応答受信   Response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？  Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	if (Hit_mode == false)
	{
		// Gコマンド送信   G command transmission
		if (SGWrite(_Go()) == false)
		{
			return false;
		}

		// 応答受信 Response received
		sss = String::Empty;
		if (SGRead(sss) == false)
		{
			return false;
		}

		// 応答がOKか？  Is the response OK?
		if (sss->IndexOf("OK") < 0)
		{
			return false;
		}
	}

	// Ready確認   Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;

}

// 移動  Movement (Absolute)
bool SKSampleClass::MoveA(int Axis, int Vdata)
{
	String^ sss;

	// Mコマンド送信  Send M command
	if (SGWrite(_MoveAbsolutely(Axis, Vdata)) == false)
	{
		return false;
	}

	// 応答受信   Response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？  Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	if (Hit_mode == false)
	{
		// Gコマンド送信   G command transmission
		if (SGWrite(_Go()) == false)
		{
			return false;
		}

		// 応答受信 Response received
		sss = String::Empty;
		if (SGRead(sss) == false)
		{
			return false;
		}

		// 応答がOKか？  Is the response OK?
		if (sss->IndexOf("OK") < 0)
		{
			return false;
		}
	}

	// Ready確認   Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;

}

// 論理原点設定  Logical origin setting
bool SKSampleClass::ResetPosition(int Axis)
{
	String^ sss;
	

	// Ready確認  Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	// Rコマンド送信    R command transmission
	if (SGWrite(_ReturnLogicalOrigin(Axis)) == false)
	{
		return false;
	}

	// 応答受信   Response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？  Check if response OK
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	// Ready確認   Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;

}

//論理原点復帰   Logical origin return
bool SKSampleClass::ReturnLogicalOrigin(int Axis, int cPos)
{
	String^ sss;

	// Ready確認   Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	// Mコマンド送信   Send M command
	if (SGWrite(_MoveRelativity(Axis, cPos * -1)) == false)
	{
		return false;
	}

	// 応答受信   Response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？   Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	// Gコマンド送信  G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}

	// 応答受信 response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？ Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	// Ready確認   Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;

}

// 減速停止  Deceleration stop
bool SKSampleClass::StopStage(int Axis)
{
	String^ sss;

	// Lｺﾏﾝﾄﾞ送信 Send L command
	if (SGWrite(_StopStage(Axis)) == false)
	{
		return false;
	}

	// 応答受信  response received 
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？ Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	return true;
}
// 非常停止   emergency stop
bool SKSampleClass::StopStageEmergency()
{
	String^ sss;

	// Lｺﾏﾝﾄﾞ送信  Send L command
	if (SGWrite(_StopStageEmergency()) == false)
	{
		return false;
	}

	// 応答受信  response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？ Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	return true;

}

// ﾎﾟｼﾞｼｮﾝ位置書込み(GIP101)  Set Position Position (GIP101)
bool SKSampleClass::SetPosition(int PNo, int Value)
{
	String^ sss;

	// Pコマンド送信  P command transmission
	if (SGWrite(_SetPosition(PNo, Value)) == false)
	{
		return false;
	}

	// 応答受信  response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？   Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	return true;

}

// ﾎﾟｼﾞｼｮﾝ位置移動(GIP101)  Position position movement (GIP101)
bool SKSampleClass::MovePosition(int PNo)
{
	String^ sss;



	// Bコマンド送信  B command transmission
	if (SGWrite(_MovePosition(PNo)) == false)
	{
		return false;
	}

	// 応答受信  response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？  Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	// Gコマンド送信  G command transmission
	if (SGWrite(_Go()) == false)
	{
		return false;
	}

	// 応答受信 response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return false;
	}

	// 応答がOKか？  Is the response OK?
	if (sss->IndexOf("OK") < 0)
	{
		return false;
	}

	// Ready確認  Ready confirmation
	if (Wait_Ready() == false)
	{
		return false;
	}

	return true;

}

// ﾎﾟｼﾞｼｮﾝ位置取得(GIP101)  Position position acquisition (GIP101)
String^ SKSampleClass::StatusP(int PNo)
{
	String^ sss;

	// ?コマンド送信  '? Command transmission
	if (SGWrite(_Status3(PNo.ToString())) == false)
	{
		return String::Empty;
	}

	// 応答受信  response received
	sss = String::Empty;
	if (SGRead(sss) == false)
	{
		return String::Empty;
	}

	return sss;

}

// Hit mode Command
String^ SKSampleClass::_HitMode()
{

	return "Z:1";

}
// Shot Mode 
String^ SKSampleClass::_ShotMode()
{

	return "Z:0";
}
