﻿#pragma once
//----------------------------------------------------------------------------------------------
//	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
//	This class can be used by anyone provided that the copyright notice remains intact.
//	Program developed by ABED Toufik, checked and confirmed by K.Nobatake.
//
//	This class helper is used to operate SIGMAKOKI Controllers (GSC01, GIP-101B, SHOT302/304GS, Hit, HSC-103, PGC-04).
//	Use the class to build your own program.
//	We can provide support for the other controllers.
//	Contact us : Phone (+ 81-3-5638-8228) or sales@sigma-koki.com.
//----------------------------------------------------------------------------------------------

#include <string>
#include <vector>
#include <cmath>
#include <stdexcept>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Windows::Forms;
using namespace System::IO::Ports;
using namespace System::Threading;

public ref class SKSampleClass
{
public:
	enum class Controller_Type
	{
		SHOT_302GS,
		SHOT_304GS,
		GIP_101B,
		GSC01,
		PGC_04,
		HSC_103,
		Hit_MV
	};

public:
	SerialPort^ SK_SerialPort;
private:
	Controller_Type Controller = static_cast<Controller_Type>(0);
	int MaxAxis = 0;
	String^ ErrMsg;
	String^ ErrTitle;
	Label^ PosText;
public:
	bool Language = false;
	bool Hit_mode;

	virtual ~SKSampleClass()
	{
		delete SK_SerialPort;
		delete PosText;
	}

	SKSampleClass();


	// Controller Type
	Controller_Type getControllerType() ;

	void setControllerType(Controller_Type value);

	// PortName
	String ^ getPortName() ;

	void setPortName(String^ &value);

	// Baudrate
	int getBaudrate() ;

	void setBaudrate(int value);

	// Delimiter
	String ^ getDelimiter();

	void setDelimiter(String^ &value);

	// Timeout
	int getTimeout() ;

	void setTimeout(int value);

	// PortNames
	array<Object^>^ GetPortNames();


	// test the conection succes 
	bool ConnectTest();

	bool PortClose();

	// Wait to Ready
	bool Wait_Ready();

	// Get Label postion
	Label ^getCurPosLabel() ;

	// set Label Position 
	void setCurPosLabel(Label ^value);

	// 現在位置   Current position
	int GetPosition();


	// 機械原点復帰  Mechanical origin return
	bool ReturnOrigin(int Axis);

	// 移動（相対）  Movement (relative)
	bool MoveR(int Axis, int Vdata);
	// 移動（Absol）  Movement (Absolute)
	bool MoveA(int Axis, int Vdata);
//
	// 論理原点設定  Logical origin setting
	bool ResetPosition(int Axis);
//
	 //論理原点復帰   Logical origin return
	bool ReturnLogicalOrigin(int Axis, int cPos);
//
	// 減速停止  Deceleration stop
	bool StopStage(int Axis);
//
	// 非常停止   emergency stop
	bool StopStageEmergency();
//
	// ﾎﾟｼﾞｼｮﾝ位置書込み(GIP101)  Write Position Position (GIP101)
	bool SetPosition(int PNo, int Value);
//
	// ﾎﾟｼﾞｼｮﾝ位置移動(GIP101)  Position position movement (GIP101)
	bool MovePosition(int PNo);
//
	// ﾎﾟｼﾞｼｮﾝ位置取得(GIP101)  Position position acquisition (GIP101)
	String^ StatusP(int PNo);
//
	bool SGOpen();
//
//
//	// Close
	bool SGClose();
//
//
	// Write
	bool SGWrite( String^ Wcmd);
//
//
	// Read
	bool SGRead(String^ &Rcmd);
//
// Hit Mode
	bool HitMode();
// Shot Mode
	bool ShotMode();
//
	// 移動方向（内部関数）Moving direction (internal function)
private:
	String^ GetAxisDir(int Value);


	// 移動方向2（内部関数） Moving direction 2 (internal function)
	String^ GetAxisDir2(bool Value);


	// Qｺﾏﾝﾄﾞから座標値を取出す   Extract coordinate values from Q command
	String^ inGet_Coordinate_Value(String^ Qstatus, int AxesNum, int Axis);
//
//
	// Qｺﾏﾝﾄﾞからｽﾃｰﾀｽを取出す  Extract status from Q-command
	String^ inGet_Stage_Status(String^ Qstatus, int AxesNum, int ACKNumber);
	String^ get_status_hit(int Axis);
//
//
	// Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command
	int Get_Qlen(int AxesNum);
// hit mode 
	String^ _HitMode();
	

	// Shot mode 
	String^ _ShotMode();
	
//
	// Return origin
	String^ _ReturnOrigin(int Axis);
//
//
	// Move relativity
	String^ _MoveRelativity(int Axis, int Value1);
//
//
	// Move absolutely
	String^ _MoveAbsolutely(int Axis, int Value1);
//
//
	// Move with JOG
	String^ _Jog(int Axis, bool Dir_Renamed);

//
	// Drive
	String^ _Go();
//
//
	// Return logical origin
	String^ _ReturnLogicalOrigin(int Axis);
//
//
	// Move Position
	String^ _MovePosition(int BNo);
//
//
	// Set Position
	String^ _SetPosition(int BNo, int Value1);

//
	// Emergency stop
	String^ _StopStageEmergency();
//
//
	// Stop
	String^ _StopStage(int Axis);
//
//
	// Speed
	String^ _Speed(int Axis, int Slow, int Fast, int Rate_Renamed);
//
//
	// Get the status "Q"
	String ^ _StatusQ();
//
//
	// Get the status "!"
	String^ _Status2();
//
	// Get the status "?"
public:
	String^ _Status3(String^ Para);

	// Get the coordinate value from the status "Q"
private:
	String^  Get_Coordinate_Value(String^  Qstatus, int Axis );

	// Get the status from the status "Q"
	String^ Get_Stage_Status(String^ Qstatus, int ACKNumber, int Axis );
};
