﻿'-----------------------------------------------------------------------------------------------
'	Copyright © 2020 SIGMAKOKI Co.,LTD. All right reserved.
'	This class can be used by anyone provided that the copyright notice remains intact.
'	Program developed by ABED Toufik, Updated and confirmed by K.Nobatake.
'
'	This class helper is used to operate SIGMAKOKI Controllers (GSC01, GIP-101B, SHOT302/304GS, Hit, HSC-103, PGC-04).
'	Use the class to build your own program.
'	We can provide support for the other controllers.
'	Contact us : Phone(+81 - 3 - 5638 - 8228) Or sales@sigma-koki.com.
'-----------------------------------------------------------------------------------------------
Imports System.IO.Ports
Imports System.Threading

Public Class SKSampleClass
    Public Enum Controller_Type
        SHOT_304GS
        Hit_MV
        PGC_04
        SHOT_302GS
        GIP_101B
        HSC_103
        GSC01
    End Enum

    Dim SK_SerialPort As SerialPort
    Dim Controller As Controller_Type
    Dim MaxAxis As Integer
    Dim ErrMsg As String
    Dim ErrTitle As String
    Dim PosText As Label
    Public Language As Boolean
    Public Hit_mode As Boolean


    Public Sub New()
        SK_SerialPort = New SerialPort
        SK_SerialPort.PortName = "COM1"     'Default value
        SK_SerialPort.BaudRate = 9600       'Default value
        SK_SerialPort.NewLine = vbCrLf      'Default value
        SK_SerialPort.ReadTimeout = 5000    'Default value
        SK_SerialPort.RtsEnable = True
        ControllerType = Controller_Type.SHOT_304GS 'Default value
    End Sub


    'Controller Type
    Public Property ControllerType As Controller_Type
        Get
            Return Controller
        End Get
        Set(value As Controller_Type)
            Controller = value
            If Controller = Controller_Type.SHOT_302GS Then
                MaxAxis = 2
            ElseIf Controller = Controller_Type.GIP_101B OrElse Controller = Controller_Type.GSC01 Then
                MaxAxis = 1
            ElseIf Controller = Controller_Type.HSC_103 Then
                Hit_mode = True
                MaxAxis = 3
            ElseIf Controller = Controller_Type.PGC_04 Then
                Hit_mode = True
                MaxAxis = 4
            ElseIf Controller = Controller_Type.Hit_MV Then
                Hit_mode = True
                MaxAxis = 4
            Else
                MaxAxis = 4
            End If
        End Set
    End Property

    'PortName
    Public Property PortName As String
        Get
            Return SK_SerialPort.PortName
        End Get
        Set(value As String)
            SK_SerialPort.PortName = value
        End Set
    End Property

    'Baudrate
    Public Property Baudrate As Integer
        Get
            Return SK_SerialPort.BaudRate
        End Get
        Set(value As Integer)
            SK_SerialPort.BaudRate = value
        End Set
    End Property

    'Delimiter
    Public Property Delimiter As String
        Get
            Return SK_SerialPort.NewLine
        End Get
        Set(value As String)
            SK_SerialPort.NewLine = value
        End Set
    End Property

    'Timeout
    Public Property Timeout As Integer
        Get
            Return SK_SerialPort.ReadTimeout
        End Get
        Set(value As Integer)
            SK_SerialPort.ReadTimeout = value
        End Set
    End Property

    'PortNames
    Public Function GetPortNames() As String()
        Dim sss As String()
        sss = SerialPort.GetPortNames()
        Return sss
    End Function


    'test the conection succes 
    Public Function ConnectTest() As Boolean
        Dim sss As String
        Dim st As String
        Dim WkStr As String

        Try

            'デバイスOPEN  device open
            If SGOpen() = False Then
                Return False
            End If

            SK_SerialPort.DiscardInBuffer()  'ﾊﾞｯﾌｧｸﾘｱ-->GIP101の場合,最初にｺﾞﾐが入る  In case of buffer clear-> GIP101, the level is entered first

            'Qｺﾏﾝﾄﾞ送信 Send Q command
            If SGWrite(_StatusQ()) = False Then
                Return False
            End If

            '応答受信 response received
            sss = String.Empty

            If SGRead(sss) = False Then
                Return False
            End If

            '受信文字確認  Check received characters
            WkStr = Get_Stage_Status(sss, 3, MaxAxis)
            st = WkStr.ToUpper()

            If Equals(st, "R") OrElse Equals(st, "B") Then
                Return True
            Else
                Return False
            End If

        Catch __unusedException1__ As Exception
            Return False
        End Try
    End Function

    Public Function PortClose() As Boolean
        Dim rtn As Boolean
        rtn = SGClose()
        Return rtn
    End Function

    'Wait to Ready
    Public Function Wait_Ready() As Boolean
        Dim sss As String
        Dim st As String

        Try
            Do
                System.Windows.Forms.Application.DoEvents()

                If SGWrite(_StatusQ()) = False Then
                    Return False
                End If

                '応答受信  response received
                sss = Nothing

                If SGRead(sss) = False Then
                    Return False
                End If

                If Not Equals(sss, "") Then
                    If Not PosText Is Nothing Then
                        CurPosLabel.Text = Get_Coordinate_Value(sss, 1)
                    End If

                    If Hit_mode = True Then
                        st = get_status_hit(1)
                        If Equals(st, "0") Then
                            Return True
                        End If
                    Else
                        st = Get_Stage_Status(sss, 3, MaxAxis).ToUpper()

                        If Equals(st, "R") Then
                            Return True
                        End If
                    End If

                End If

            Loop While True

        Catch ex As Exception
            Return False
        End Try

        Return True
    End Function


    Public Property CurPosLabel As Label
        Get
            Return PosText
        End Get
        Set(value As Label)
            PosText = value
        End Set
    End Property

    '現在位置   Current position
    Public Function GetPosition() As Integer
        Dim sss As String
        Dim pos As String

        'Ready確認 Confirmation  
        If Wait_Ready() = False Then
            Return 0
        End If

        'Qコマンド送信  Q command transmission
        If SGWrite(_StatusQ()) = False Then
            Return 0
        End If

        sss = Nothing

        If SGRead(sss) = False Then  '応答受信   Response received
            Return 0
        End If

        pos = Get_Coordinate_Value(sss, MaxAxis) '座標値を得る  Get coordinate values
        Return Integer.Parse(pos)
    End Function


    '機械原点復帰  Mechanical origin return
    Public Function ReturnOrigin(Axis As Integer) As Boolean
        Dim sss As String

        'Ready確認  Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        'Hｺﾏﾝﾄﾞ送信    Send H command
        If SGWrite(_ReturnOrigin(Axis)) = False Then
            Return False
        End If

        '応答受信    response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？ Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If


        'Ready確認  Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        Return True
    End Function

    'hit Mode 
    Public Function HitMode() As Boolean
        Dim sss As String

        'Zｺﾏﾝﾄﾞ送信    Send Z command
        If SGWrite(_HitMode()) = False Then
            Return False
        End If

        '応答受信    response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？ Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If


        'Ready確認  Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        Return True
    End Function

    'Shot Mode 
    Public Function ShotMode() As Boolean
        Dim sss As String

        'Zｺﾏﾝﾄﾞ送信    Send Z command
        If SGWrite(_SHotMode()) = False Then
            Return False
        End If

        '応答受信    response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？ Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If


        'Ready確認  Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        Return True
    End Function

    '移動（相対）  Movement (relative)
    Public Function MoveR(Axis As Integer, ByVal Vdata As Integer) As Boolean
        Dim sss As String

        'Mコマンド送信  Send M command
        If SGWrite(_MoveRelativity(Axis, Vdata)) = False Then
            Return False
        End If

        '応答受信   Response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？  Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        If Hit_mode = False Then
            'Gコマンド送信   G command transmission
            If SGWrite(_Go()) = False Then
                Return False
            End If

            '応答受信 Response received
            sss = Nothing
            If SGRead(sss) = False Then
                Return False
            End If

            '応答がOKか？  Is the response OK?
            If sss.IndexOf("OK") < 0 Then
                Return False
            End If
        End If

        'Ready確認   Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If
        Return True
    End Function

    '移動（相対）  Movement (Absolute)
    Public Function MoveA(Axis As Integer, ByVal Vdata As Integer) As Boolean
        Dim sss As String

        'Aコマンド送信  Send A command
        If SGWrite(_MoveAbsolutely(Axis, Vdata)) = False Then
            Return False
        End If

        '応答受信   Response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？  Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        If Hit_mode = False Then
            'Gコマンド送信   G command transmission
            If SGWrite(_Go()) = False Then
                Return False
            End If

            '応答受信 Response received
            sss = Nothing
            If SGRead(sss) = False Then
                Return False
            End If

            '応答がOKか？  Is the response OK?
            If sss.IndexOf("OK") < 0 Then
                Return False
            End If
        End If

        'Ready確認   Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If
        Return True
    End Function

    '論理原点設定  Logical origin setting
    Public Function ResetPosition(Axis As Integer) As Boolean
        Dim sss As String

        'Ready確認  Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        'Rコマンド送信    R command transmission
        If SGWrite(_ReturnLogicalOrigin(Axis)) = False Then
            Return False
        End If

        '応答受信   Response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？  Check if response OK
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        'Ready確認   Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        Return True
    End Function

    '論理原点復帰   Logical origin return
    Public Function ReturnLogicalOrigin(Axis As Integer, ByVal Optional cPos As Integer = 0) As Boolean
        Dim sss As String

        'Ready確認   Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        'Mコマンド送信   Send M command
        If SGWrite(_MoveRelativity(Axis, cPos * -1)) = False Then
            Return False
        End If

        '応答受信   Response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？   Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        'Gコマンド送信  G command transmission
        If SGWrite(_Go()) = False Then
            Return False
        End If

        '応答受信 response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？ Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        'Ready確認   Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        Return True
    End Function

    '減速停止  Deceleration stop
    Public Function StopStage(Axis As Integer) As Boolean
        Dim sss As String

        'Lｺﾏﾝﾄﾞ送信 Send L command
        If SGWrite(_StopStage(Axis)) = False Then
            Return False
        End If

        '応答受信  response received 
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？ Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        Return True
    End Function

    '非常停止   emergency stop
    Public Function StopStageEmergency() As Boolean
        Dim sss As String

        'Lｺﾏﾝﾄﾞ送信  Send L command
        If SGWrite(_StopStageEmergency()) = False Then
            Return False
        End If

        '応答受信  response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？ Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        Return True
    End Function

    'ﾎﾟｼﾞｼｮﾝ位置書込み(GIP101)  Write Position Position (GIP101)
    Public Function SetPosition(ByVal PNo As Integer, ByVal Value As Integer) As Boolean
        Dim sss As String

        'Pコマンド送信  P command transmission
        If SGWrite(_SetPosition(PNo, Value)) = False Then
            Return False
        End If

        '応答受信  response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？   Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        Return True
    End Function

    'ﾎﾟｼﾞｼｮﾝ位置移動(GIP101)  Position position movement (GIP101)
    Public Function MovePosition(ByVal PNo As Integer) As Boolean
        Dim sss As String



        'Bコマンド送信  B command transmission
        If SGWrite(_MovePosition(PNo)) = False Then
            Return False
        End If

        '応答受信  response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？  Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        'Gコマンド送信  G command transmission
        If SGWrite(_Go()) = False Then
            Return False
        End If

        '応答受信 response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return False
        End If

        '応答がOKか？  Is the response OK?
        If sss.IndexOf("OK") < 0 Then
            Return False
        End If

        'Ready確認  Ready confirmation
        If Wait_Ready() = False Then
            Return False
        End If

        Return True
    End Function

    'ﾎﾟｼﾞｼｮﾝ位置取得(GIP101)  Position position acquisition (GIP101)
    Public Function StatusP(ByVal PNo As Integer) As String
        Dim sss As String

        '?コマンド送信  '? Command transmission
        If SGWrite(_Status3(PNo.ToString())) = False Then
            Return Nothing
        End If

        '応答受信  response received
        sss = Nothing

        If SGRead(sss) = False Then
            Return Nothing
        End If

        Return sss
    End Function


    Public Function SGOpen() As Boolean
        Try

            If SK_SerialPort.IsOpen = False Then
                SK_SerialPort.Open()
            End If

            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "デバイスOPEN時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the device open."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function


    'Close
    Public Function SGClose() As Boolean
        Try

            If SK_SerialPort.IsOpen = True Then
                SK_SerialPort.Close()
            End If

            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "デバイスCLOSE時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the device close."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function


    'Write
    Public Function SGWrite(ByVal Wcmd As String) As Boolean
        Try



            If SK_SerialPort.CtsHolding <> True Then

                If Language = True Then
                    ErrMsg = "送信時にエラーが発生しました。"
                    ErrTitle = "通信エラー"
                Else
                    ErrMsg = "Error occurred at the time of the transmission."
                    ErrTitle = "Communication Error"
                End If

                MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return False
            End If

            SK_SerialPort.WriteLine(Wcmd)

            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "送信時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the transmission."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return False
        End Try
    End Function


    'Read
    Public Function SGRead(ByRef Rcmd As String) As Boolean

        Try
            Rcmd = SK_SerialPort.ReadLine()

            Return True
        Catch __unusedException1__ As Exception

            If Language = True Then
                ErrMsg = "受信時にエラーが発生しました。"
                ErrTitle = "通信エラー"
            Else
                ErrMsg = "Error occurred at the time of the reception."
                ErrTitle = "Communication Error"
            End If

            MessageBox.Show(ErrMsg, ErrTitle, MessageBoxButtons.OK, MessageBoxIcon.[Error])
            Return False
        End Try
    End Function




    '移動方向（内部関数）Moving direction (internal function)
    Private Function GetAxisDir(ByVal Value As Integer) As String
        If Value < 0 Then
            Return "-"
        Else
            Return "+"
        End If
    End Function


    '移動方向2（内部関数） Moving direction 2 (internal function)
    Private Function GetAxisDir2(ByVal Value As Boolean) As String
        If Value = False Then
            Return "-"
        Else
            Return "+"
        End If
    End Function


    'Qｺﾏﾝﾄﾞから座標値を取出す   Extract coordinate values from Q command
    Private Function inGet_Coordinate_Value(ByVal Qstatus As String, ByVal AxesNum As Integer, ByVal Axis As Integer) As String
        Dim stp As Integer
        Dim AxLen As Integer
        Dim BeforeStr As String
        Dim AfterStr As String
        AxLen = Get_Qlen(AxesNum) '文字列長    String length

        If Hit_mode = True Then
            Select Case Axis
                Case 4
                    stp = 3
                Case 3
                    stp = 2
                Case 2
                    stp = 1
                Case Else
                    stp = 0
            End Select
            Dim kk = Qstatus.Length
            BeforeStr = Qstatus.Substring(stp, kk - stp)
            BeforeStr = BeforeStr.Substring(0, BeforeStr.IndexOf(","))
            AfterStr = BeforeStr.Replace(" ", "")
            Return AfterStr
        Else
            'Qｺﾏﾝﾄﾞの長さ   Q command length
            If Qstatus.Length < AxLen Then
                Return "0"
            End If

            '取出す先頭  Retrieving head
            Select Case Axis
                Case 4
                    stp = 34
                Case 3
                    stp = 23
                Case 2
                    stp = 12
                Case Else
                    stp = 1
            End Select

            BeforeStr = Qstatus.Substring(stp - 1, 10) '-1:1文字目のindexは0のため  -1: Since the index of the first character is 0
            AfterStr = BeforeStr.Replace(" ", "")
            Return AfterStr
            '		    return Qstatus.Substring(stp - 1, 10); //-1:1文字目のindexは0のため

        End If


    End Function


    'Qｺﾏﾝﾄﾞからｽﾃｰﾀｽを取出す  Extract status from Q-command
    Private Function inGet_Stage_Status(ByVal Qstatus As String, ByVal AxesNum As Integer, ByVal ACKNumber As Integer) As String
        Dim stp As Integer
        Dim AxLen As Integer
        AxLen = Get_Qlen(AxesNum) '文字列長  String length
        'Qｺﾏﾝﾄﾞの長さ  Q command length
        If Qstatus.Length < AxLen Then
            Return "0"
        End If

        '取出す先頭  Retrieving head
        Select Case AxesNum
            Case 4   '4軸    4axis
                Select Case ACKNumber
                    Case 1
                        stp = 45
                    Case 2
                        stp = 47
                    Case Else
                        stp = 49
                End Select

            Case 3  '3軸 3axis
                Select Case ACKNumber
                    Case 1
                        stp = 34
                    Case 2
                        stp = 36
                    Case Else
                        stp = 38
                End Select

            Case 2  '2軸 2axis
                Select Case ACKNumber
                    Case 1
                        stp = 23
                    Case 2
                        stp = 25
                    Case Else
                        stp = 27
                End Select '1軸  1axis
            Case Else

                Select Case ACKNumber
                    Case 1
                        stp = 12
                    Case 2
                        stp = 14
                    Case Else
                        stp = 16
                End Select
        End Select

        Return Qstatus.Substring(stp - 1, 1) '-1:1文字目のindexは0のため   -1: Since the index of the first character is 0
    End Function


    'Qｺﾏﾝﾄﾞの返信文字列長   Reply string length of Q command
    Private Function Get_Qlen(ByVal AxesNum As Integer) As Integer
        Select Case AxesNum '軸数  Number of axes
            Case 4
                Return 49
            Case 3
                Return 38
            Case 2
                Return 27
            Case 1
                Return 16
            Case Else
                Return 0
        End Select
    End Function


    'Return origin
    Private Function _ReturnOrigin(ByVal Axis As Integer) As String

        If Hit_mode = True Then
            If Axis = 0 OrElse Axis = 1 Then  'All axis
                Return "H:1"
            ElseIf Axis = 2 Then  'axis 2
                Return "H:,1"
            ElseIf Axis = 3 Then  'axis 3
                Return "H:,,1"
            ElseIf Axis = 4 Then  'axis 4
                Return "H:,,,1"
            Else
                Return ""
            End If
        Else

            If Axis = 0 Then  'All axis
                Return "H:1"
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  'Each axis
                Return "H:" & Axis.ToString()
            Else
                Return ""
            End If
        End If

    End Function


    'Move relativity
    Private Function _MoveRelativity(ByVal Axis As Integer, ByVal Value1 As Integer) As String
        Dim WkStr As String
        If Hit_mode = True Then
            If Axis = 0 OrElse Axis = 1 Then
                WkStr = "M:" & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis = 2 Then
                WkStr = "M:," & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis = 3 Then
                WkStr = "M:,," & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis = 4 Then
                WkStr = "M:,,," & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            Else
                Return ""
            End If
        Else

            If Axis = 0 Then  'All axis
                WkStr = "M:1" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  'Each axis
                WkStr = "M:" & Axis.ToString() & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
                Return WkStr
            Else
                Return ""
            End If
        End If
    End Function


    'Move absolutely
    Private Function _MoveAbsolutely(ByVal Axis As Integer, ByVal Value1 As Integer) As String
        Dim WkStr As String
        If Hit_mode = True Then
            If Axis = 0 OrElse Axis = 1 Then
                WkStr = "A:" & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis = 2 Then
                WkStr = "A:," & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis = 3 Then
                WkStr = "A:,," & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis = 4 Then
                WkStr = "A:,,," & GetAxisDir(Value1) & Math.Abs(Value1).ToString()
                Return WkStr
            Else
                Return ""
            End If
        Else
            If Axis = 0 Then  'All axis
                WkStr = "A:1" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
                Return WkStr
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  'Each axis
                WkStr = "A:" & Axis.ToString() & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
                Return WkStr
            Else
                Return ""
            End If
        End If
    End Function


    'Move with JOG
    Private Function _Jog(ByVal Axis As Integer, ByVal Dir_Renamed As Boolean) As String
        Dim WkStr As String
        If Hit_mode = True Then

            If Axis = 0 OrElse Axis = 1 Then  ' All axis
                WkStr = "J:" & GetAxisDir2(Dir_Renamed)
                Return WkStr
            ElseIf Axis = 2 Then  ' All axis
                WkStr = "J:," & GetAxisDir2(Dir_Renamed)
                Return WkStr
            ElseIf Axis = 3 Then  ' All axis
                WkStr = "J:,," & GetAxisDir2(Dir_Renamed)
                Return WkStr
            ElseIf Axis = 4 Then  ' All axis
                WkStr = "J:,,," & GetAxisDir2(Dir_Renamed)
                Return WkStr
            Else
                Return ""
            End If
        Else

            If Axis = 0 Then  ' All axis
                WkStr = "J:1" & GetAxisDir2(Dir_Renamed)
                Return WkStr
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  ' Each axis
                WkStr = "J:" & Axis.ToString() + GetAxisDir2(Dir_Renamed)
                Return WkStr
            Else
                Return ""
            End If
        End If

    End Function


    'Drive
    Private Function _Go() As String
        Return "G:"
    End Function


    'Return logical origin
    Private Function _ReturnLogicalOrigin(ByVal Axis As Integer) As String

        If Hit_mode = True Then
            If Axis = 0 OrElse Axis = 1 Then  'Axis 1
                Return "R:1"
            ElseIf Axis = 2 Then
                Return "R:,1"
            ElseIf Axis = 3 Then
                Return "R:,,1"
            ElseIf Axis = 4 Then
                Return "R:,,,1"
            Else
                Return ""
            End If
        Else

            If Axis = 0 OrElse Axis = 1 Then  'Axis 1
                Return "R:1"
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  'Each axis
                Return "R:" & Axis.ToString()
            Else
                Return ""
            End If
        End If
    End Function


    'Move Position
    Private Function _MovePosition(ByVal BNo As Integer) As String
        If BNo = 0 Then  'All axis
            Return "B:1"
        ElseIf BNo >= 1 AndAlso BNo <= 5 Then  '1-5
            Return "B:" & BNo.ToString()
        Else
            Return ""
        End If
    End Function


    'Set Position
    Private Function _SetPosition(ByVal BNo As Integer, ByVal Value1 As Integer) As String
        Dim WkStr As String

        If BNo = 0 Then  'All axis
            WkStr = "P:B1" & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
            Return WkStr
        ElseIf BNo >= 1 AndAlso BNo <= 5 Then  '1-5
            WkStr = "P:B" & BNo.ToString() & GetAxisDir(Value1) & "P" & Math.Abs(Value1).ToString()
            Return WkStr
        Else
            Return ""
        End If
    End Function


    'Emergency stop
    Private Function _StopStageEmergency() As String
        Return "L:E"
    End Function


    'Stop
    Private Function _StopStage(ByVal Axis As Integer) As String
        If Hit_mode = True Then
            If Axis = 0 Then  'Axis 1
                Return "L:1"
            ElseIf Axis = 2 Then
                Return "L:,1"
            ElseIf Axis = 3 Then
                Return "L:,,1"
            ElseIf Axis = 4 Then
                Return "L:,,,1"
            Else
                Return ""
            End If
        Else

            If Axis = 0 Then  'All axis
                Return "L:1"
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  'Each axis
                Return "L:" & Axis.ToString()
            Else
                Return ""
            End If
        End If

    End Function


    'Speed
    Private Function _Speed(ByVal Axis As Integer, ByVal Slow As Integer, ByVal Fast As Integer, ByVal Rate_Renamed As Integer) As String
        Dim WkStr As String
        If Hit_mode = True Then

            If Slow < 1 OrElse Slow > 999999999 OrElse Fast < 1 OrElse Fast > 999999999 OrElse Rate_Renamed < 0 OrElse Rate_Renamed > 1000 Then
                Return ""
            End If

            If Axis = 0 OrElse Axis = 1 Then  ' first axis
                WkStr = "D:1" & Math.Abs(Slow).ToString() + Math.Abs(Fast).ToString() + Math.Abs(Rate_Renamed).ToString()
                Return WkStr
            End If

            If Axis = 2 OrElse Axis <= MaxAxis Then  ' All axis
                WkStr = "D:" & Axis.ToString() + Math.Abs(Slow).ToString() + Math.Abs(Fast).ToString() + Math.Abs(Rate_Renamed).ToString()
                Return WkStr
            Else
                Return ""
            End If
        Else



            If Slow < 1 OrElse Slow > 500000 OrElse Fast < 1 OrElse Fast > 500000 OrElse Rate_Renamed < 0 OrElse Rate_Renamed > 1000 Then
                Return ""
            End If

            If Axis = 0 Then  'All axis
                WkStr = "D:W" & "S" & Math.Abs(Slow).ToString() & "F" & Math.Abs(Fast).ToString() & "R" & Math.Abs(Rate_Renamed).ToString()
                Return WkStr
            ElseIf Axis >= 1 AndAlso Axis <= MaxAxis Then  'Each axis
                WkStr = "D:" & Axis.ToString() & "S" & Math.Abs(Slow).ToString() & "F" & Math.Abs(Fast).ToString() & "R" & Math.Abs(Rate_Renamed).ToString()
                Return WkStr
            Else
                Return ""
            End If
        End If

    End Function

    'Set Speed
    Public Function Speed(ByVal Axis As Integer, ByVal Slow As Integer, ByVal Fast As Integer, ByVal Rate_Renamed As Integer) As Boolean

        If SGWrite(_Speed(Axis, Slow, Fast, Rate_Renamed)) Then
            Return True
        Else
            Return False
        End If


    End Function
    'Get the status "Q"
    Private Function _StatusQ() As String
        Return "Q:"
    End Function


    'Get the status "!"
    Private Function _Status2() As String
        Return "!:"
    End Function

    'Get the status "?"
    Public Function _Status3(ByVal Para As String) As String
        Return "?:" & Para
    End Function

    'Get the coordinate value from the status "Q"
    Private Function Get_Coordinate_Value(ByVal Qstatus As String, ByVal Optional Axis As Integer = 1) As String
        Return inGet_Coordinate_Value(Qstatus, 1, 1)
    End Function


    'Get the status from the status "Q"
    Private Function Get_Stage_Status(ByVal Qstatus As String, ByVal ACKNumber As Integer, ByVal Optional Axis As Integer = 1) As String
        Return inGet_Stage_Status(Qstatus, Axis, ACKNumber)
    End Function

    'get Status Hit

    Private Function get_status_hit(ByVal Axis As Integer) As String
        Dim Wkl As String
        Wkl = String.Empty
        SGWrite(_Status2())
        Thread.Sleep(100)
        SGRead(Wkl)
        Dim stp As Integer

        Select Case Axis
            Case 4
                stp = 3
            Case 3
                stp = 2
            Case 2
                stp = 1
            Case Else
                stp = 0
        End Select

        Dim kk As Integer
        kk = Wkl.Length
        Wkl = Wkl.Substring(stp, kk - stp)
        Wkl = Wkl.Substring(0, Wkl.IndexOf(","))
        Return Wkl

    End Function


    'Hit Mode 
    Private Function _HitMode() As String
        Return "Z:1"
    End Function

    'Shot Mode
    Private Function _SHotMode() As String
        Return "Z:0"
    End Function

End Class
